/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	SPI.c
  \brief    	Driver to transmit and receive strings via SPI
  \author   	Freescale Semiconductor
  \author   	Guadalajara Applications Laboratory RTAC Americas
  \author      Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

/************************************************************************************************/
/*                                      Includes Section                                        */
/************************************************************************************************/
#include "SPI.h"

/************************************************************************************************/
/*                                     Constants Section                                        */
/************************************************************************************************/
#pragma CONST_SEG DEFAULT

/************************************************************************************************/
/*                                Function Prototipes Section                                   */
/************************************************************************************************/
#pragma CODE_SEG DEFAULT


/************************************************************************************************/
/*                                Global Variables Section                                      */
/************************************************************************************************/
#pragma DATA_SEG DEFAULT

UINT8 gau8SPI1TxBuffer[SPI1_TX_BUFF_SIZE];                       /** Array used to transmit data via SPI port */
UINT8 gau8SPI1RxBuffer[SPI1_RX_BUFF_SIZE];                       /** Array used to receive data from the SPI port */
UINT8 *gpu8SPI1TxPtr;                                            /** transmission data pointer used to send byte per byte */
UINT8 *gpu8SPI1RxPtr;                                            /** reception data pointer used to receive each incoming byte */
UINT8 gu8SPI1TxCounter;                                          /** this counter has the number of bytes to be sent */
UINT8 gu8SPI1RxCounter;                                          /** this counter has the number of bytes received */
UINT8 gu8SPI1Status;                                             /** variable used as status and control register for the SPI driver */

/************************************************************************************************/
/*                                      Functions Section                                       */
/************************************************************************************************/
#pragma CODE_SEG DEFAULT

void vfnSPI1Init(void)
{
     SPI1_SS_PIN(1);
     SPI1_SS_DDR(1);
     
     gpu8SPI1TxPtr = &gau8SPI1TxBuffer[0];                       /* Buffer Initialization */
     gpu8SPI1RxPtr = &gau8SPI1RxBuffer[0];

     SPI1_BAUD_REG=0b00000000;                                   /* Baud Rate Divisor = 2 */
	SPI1_CTR1=0b11010000;                                       /* Interrupt Enabled,Master Mode */
	                                                            /* SS as GPIO,SPSCK at the middle */
	SPI1_CTR2=0b00000000;                                       /* SS as GPIO,SPI use separate pins */
	SPI1_ST1_RX_FLAG;					                    /* Clear Interrupt Flag */
}

void vfnSPI1WriteString(UINT8 *pu8TxBuffPtr, UINT8 u8TxCounter)
{
     SPI1_SS_PIN(0);                                             /* Drive low SS */
     gu8SPI1Status |= (1<<SPI_TX_IN_PROGRESS);
     gu8SPI1TxCounter = u8TxCounter;
     gpu8SPI1TxPtr = pu8TxBuffPtr;
     SPI1_STATUS1;
     SPI1_IEN = 0;
     SPI1_DATA_REG = *gpu8SPI1TxPtr++;
     SPI1_IEN = 1;
     gu8SPI1TxCounter--;
}

void vfnSPI1ReadString(UINT8 *pu8RxBuffPtr, UINT8 u8StartAddr, UINT8 u8RxCounter)
{
     SPI1_SS_PIN(0);                                             /* Drive low SS */
     gu8SPI1Status |= (1<<SPI_RX_IN_PROGRESS);
     gpu8SPI1RxPtr = pu8RxBuffPtr;
     gu8SPI1RxCounter = u8RxCounter;
     SPI1_DATA_REG = u8StartAddr;
}

#pragma CODE_SEG __NEAR_SEG NON_BANKED
interrupt SPI1_INT_VECTOR void SPIISR(void)
{
     if (gu8SPI1Status &(1<<SPI_RX_IN_PROGRESS))
     {
          SPI1_ST1_RX_FLAG;                                      /* Clear Rx Flag */

          *gpu8SPI1RxPtr++ = SPI1_DATA_REG;                      /* Read SPI Data */
          if (gu8SPI1RxCounter)
          {
               gu8SPI1RxCounter--;
               SPI1_DATA_REG = 0x00;                             /* Dummy Tx */
          }
          else
          {
               SPI1_SS_PIN(1);                                   /* Drive high SS */
               gu8SPI1Status &= ~(1<<SPI_RX_IN_PROGRESS);
          }
     }

     if (gu8SPI1Status &(1<<SPI_TX_IN_PROGRESS))
     {
          if (SPI1_ST1_TX_FLAG)                    
          {
               SPI1_STATUS1;                                     /* Clear TX Flag */

               if (gu8SPI1TxCounter)
               {
                    gu8SPI1TxCounter--;
                    SPI1_DATA_REG = *gpu8SPI1TxPtr++;            /* Transmit next byte */
                    return;
               }
               
               SPI1_DATA_REG;
               SPI1_SS_PIN(1);                                   /* Drive high SS */
               gu8SPI1Status &= ~(1<<SPI_TX_IN_PROGRESS);
          }          
     }
}
