/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	SPI.h
  \brief    	SPI Configuration header, allows choose from different SPI channels
  \author   	Freescale Semiconductor
  \author      Guadalajara Applications Laboratory RTAC Americas     
  \author   	Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     at less 1 SPI channel must be defined

  * History:
  
*/

#ifndef __SPI1_H__               //Conditional Compilation
  #define __SPI1_H__

  /************************************************************************************************/
  /*                                      Includes Section                                        */
  /************************************************************************************************/
     #include "derivative.h" /* include peripheral declarations */
     #include "MyTypes.h"
     
     #include "SystemConfig.h"

  /************************************************************************************************/
  /*                                    Typedef Section                                           */
  /************************************************************************************************/
     typedef enum
     {
          SPI_TX_IN_PROGRESS,
          SPI_RX_IN_PROGRESS
     }_SPIFlags;

  /************************************************************************************************/
  /*                                    Defines & Macros Section                                  */
  /************************************************************************************************/


     #if (BK_SPI1_MODULE == BK_DISABLE)
          //  #define SPI0
          #define SPI1
          //  #define SPI2

          #ifdef SPI0

               #define SPI1_BAUD_REG                        (SPI0BR)
               #define SPI1_CTR1                            (SPI0C1)
               #define SPI1_IEN                             (SPI0C1_SPIE)
               #define SPI1_CTR2                            (SPI0C2)
               #define SPI1_DATA_REG                        (SPI0D)
               #define SPI1_STATUS1                         (SPI0S)
               #define SPI1_ST1_RX_FLAG                     (SPI0S_SPRF)
               #define SPI1_ST1_TX_FLAG                     (SPI0S_SPTEF)
               #define SPI1_INT_VECTOR                      (15)
               #define SPI1_SS_PIN(x)                       (PTBD_PTBD5 = (x))
               #define SPI1_SS_DDR(x)                       (PTBDD_PTBDD5 = (x))
               #define SPI1_RX_BUFF_SIZE                    (100)
               #define SPI1_TX_BUFF_SIZE                    (100)

          #else
               #ifdef SPI1

                    #define SPI1_BAUD_REG                   (SPI1BR)
                    #define SPI1_CTR1                       (SPI1C1)
                    #define SPI1_IEN                        (SPI1C1_SPIE)
                    #define SPI1_CTR2                       (SPI1C2)
                    #define SPI1_DATA_REG                   (SPI1D)
                    #define SPI1_STATUS1                    (SPI1S)
                    #define SPI1_ST1_RX_FLAG                (SPI1S_SPRF)
                    #define SPI1_ST1_TX_FLAG                (SPI1S_SPTEF)
                    #define SPI1_INT_VECTOR                 (13)
                    #define SPI1_SS_PIN(x)                  (PTBD_PTBD5 = (x))
                    #define SPI1_SS_DDR(x)                  (PTBDD_PTBDD5 = (x))
                    #define SPI1_RX_BUFF_SIZE               (100)
                    #define SPI1_TX_BUFF_SIZE               (100)

               #else
                    #ifdef SPI2
                    
                         #define SPI1_BAUD_REG              (SPI2BR)
                         #define SPI1_CTR1                  (SPI2C1)
                         #define SPI1_IEN                   (SPI2C1_SPIE)
                         #define SPI1_CTR2                  (SPI2C2)
                         #define SPI1_DATA_REG              (SPI2D)
                         #define SPI1_STATUS1               (SPI2S)
                         #define SPI1_ST1_RX_FLAG           (SPI2S_SPRF)
                         #define SPI1_ST1_TX_FLAG           (SPI2S_SPTEF)
                         #define SPI1_INT_VECTOR            (12)
                         #define SPI1_SS_PIN(x)             (PTDD_PTDD3 = (x))
                         #define SPI1_SS_DDR(x)             (PTDDD_PTDDD3 = (x))
                         #define SPI1_RX_BUFF_SIZE          (100)
                         #define SPI1_TX_BUFF_SIZE          (100)

                    #else
                         #error "You need to define a SPI to use with the Driver"
                    #endif
               #endif
          #endif
     #endif
  /************************************************************************************************/
  /*                                    Extern Variables Section                                  */
  /************************************************************************************************/
  #pragma DATA_SEG DEFAULT
  
     extern UINT8 gau8SPI1TxBuffer[SPI1_TX_BUFF_SIZE];
     extern UINT8 gau8SPI1RxBuffer[SPI1_RX_BUFF_SIZE];
     extern UINT8 gu8SPI1TxCounter;
     extern UINT8 gu8SPI1RxCounter;
     extern UINT8 gu8SPI1Status;
  /************************************************************************************************/
  /*                                  Function Prototipes Section                                 */
  /************************************************************************************************/
  #pragma CODE_SEG DEFAULT
  
     /**
     * \brief  Function to initialize the SPI communication module
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning The function is configuring the SS signal to be handled with a GPIO pin
     */
     void vfnSPI1Init(void);
     
     /**
      * \brief  Function to send strings using the SPI
      * \author Juan Czares Blanco
      * \param  UINT8* address where the data to be sent are stored
      * \param  UINT8 this parameter has the number of bytes to be sent
      * \return void
      * \todo
      * \warning
      */  
     void vfnSPI1WriteString(UINT8 *pu8TxBuffPtr, UINT8 u8TxCounter);
     
     /**
      * \brief  Function to read strings using the SPI
      * \author Juan Czares Blanco
      * \param  UINT8* address where the incoming data will be stored
      * \param  UINT8 initial address to be read
      * \param  UINT8 this parameter has the number of bytes to be read
      * \return void
      * \todo
      * \warning
      */  
     void vfnSPI1ReadString(UINT8 *pu8RxBuffPtr, UINT8 u8StartAddr, UINT8 u8RxCounter);
     
     /**
     * \brief   Interrupt vector for the SPI communication module
     * \author  Juan Czares Blanco
     * \param   void
     * \return  void
     * \todo  
     * \warning This function will be allocated in different locations depending the SPI used; see SPI_VECTOR definition
     */  
     #pragma CODE_SEG __NEAR_SEG NON_BANKED
     interrupt SPI1_INT_VECTOR void SPIISR(void);
  /************************************************************************************************/
#endif