/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	Transceiver.h
  \brief    	Transceiver Configuration header, macros to configure hardware and software
  \author   	Freescale Semiconductor
  \author      Guadalajara Applications Laboratory RTAC Americas     
  \author   	Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

#ifndef __TRANSCEIVER_H__
     #define __TRANSCEIVER_H__

     /************************************************************************************************/
     /*                                      Includes Section                                        */
     /************************************************************************************************/
     #include "SPI.h"
     #include "MyTypes.h"
     
     #include "SystemConfig.h"

     /************************************************************************************************/
     /*                                    Typedef Section                                           */
     /************************************************************************************************/

     #if (BK_TRANSCEIVER_MODULE == BK_DISABLE)
     //     #define ALWAYS_RECEIVE                                         /* Reception always on */
     //     #define ENABLE_ENDIAN                                          /* Swap payload */
     //     #define ENABLED_CODE_BYTES                                     /* Insert code bytes */
     #endif
     
     #ifdef ENABLED_CODE_BYTES
          #define MODEM_HEADER                    (4)
     #else
          #define MODEM_HEADER                    (2)
     #endif

     typedef enum
     {
          CHECK_TRANSC_ON_LINE,
          TRANSC_RELOAD_TIMER_STATE0,
          TRANSC_TX_DATA_STATE0,
          TRANSC_TX_DATA_STATE1,
          TRANSC_TX_DATA_STATE2,
          TRANSC_READ_STATUS_STATE0,
          TRANSC_READ_STATUS_STATE1,
          TRANSC_CHECK_RX_PACKET_STATE0,
          TRANSC_ENERGY_DET_STATE0,
          TRANSC_RECEIVING_STATE0,
          TRANSC_RECEIVING_STATE1,
          TRANSC_CHANGE_MODE_STATE0,
          TRANSC_CHANGE_MODE_STATE1,
          TRANSC_CHANGE_MODE_STATE2,
          TRANSC_SET_TIMEOUT_STATE0,
          TRANSC_WAIT_SPI_STATE0,
          TRANSC_ERROR_STATE0,
          TRANSC_IDLE_STATE0
     }_TransceiverMStates;
     

     typedef enum
     {
          TRANSC_IRQ_PENDING,
          TRANSC_RECEIVED_PCK,
          TRANSC_CRC_VALID,
          TRANSC_RX_FRAME_VALID,
          TRANSC_CCA_DONE,
          TRANSC_CHANNEL_CLEAR
     }_TransceiverStatusFlags;
     
     
     typedef enum
     {
          MODEM_RESET,                            /* Register 00 */
          RX_PKT_RAM,                             /* Register 01 */
          TX_PKT_RAM,                             /* Register 02 */
          TX_PKT_CTL,                             /* Register 03 */
          CCA_THRESH,                             /* Register 04 */
          IRQ_MASK,                               /* Register 05 */
          CONTROL_A,                              /* Register 06 */
          CONTROL_B,                              /* Register 07 */
          PA_ENABLE,                              /* Register 08 */
          CONTROL_C,                              /* Register 09 */
          CLKO_CTL,                               /* Register 0A */
          GPIO_DIR,                               /* Register 0B */
          GPIO_DATA_OUT,                          /* Register 0C */
          REGISTER_0D,
          REGISTER_0E,
          LO1_INT_DIV,                            /* Register 0F */
          LO1_NUM,                                /* Register 10 */
          REGISTER_11,
          PA_LVL,                                 /* Register 12 */
          REGISTER_13,
          REGISTER_14,
          REGISTER_15,
          REGISTER_16,
          REGISTER_17,
          REGISTER_18,
          REGISTER_19,
          REGISTER_1A,
          TMR_CMP1_A,                             /* Register 1B */
          TMR_CMP1_B,                             /* Register 1C */
          TMR_CMP2_A,                             /* Register 1D */
          TMR_CMP2_B,                             /* Register 1E */
          TMR_CMP3_A,                             /* Register 1F */
          TMR_CMP3_B,                             /* Register 20 */
          TMR_CMP4_A,                             /* Register 21 */
          TMR_CMP4_B,                             /* Register 22 */
          TC2_PRIME,                              /* Register 23 */
          IRQ_STATUS,                             /* Register 24 */
          RST_IND,                                /* Register 25 */
          CURRENT_TIME_A,                         /* Register 26 */
          CURRENT_TIME_B,                         /* Register 27 */
          GPIO_DATA_IN,                           /* Register 28 */
          REGISTER_29,
          REGISTER_2A,
          REGISTER_2B,
          CHIP_ID,                                /* Register 2C */
          RX_STATUS,                              /* Register 2D */
          TIMESTAMP_A,                            /* Register 2E */
          TIMESTAMP_B,                            /* Register 2F */
          BER_ENABLE,                             /* Register 30 */
          PSM_MODE                                /* Register 31 */
     }_ModemRegisters;

     typedef enum
     {
          CRC_VALID,
          CCA,
          TMR2_IRQ,
          TMR4_IRQ,
          TMR3_IRQ,
          CCA_IRQ,
          TX_SENT_IRQ,
          RX_RCVD_IRQ,
          TMR1_IRQ,
          DOZE_IRQ,
          ATTN_IRQ,
          UnusedFlag,
          STRM_DATA_ERR,
          ARB_BUSY_ERR,
          RAM_ADDR_ERR,
          PLL_LOCK_IRQ
     }_IRQFlags;
     
     typedef struct
     {
          UINT8 DDRIn  :7;
          UINT8 DDROut :7;
          UINT8 Drv    :2;
     }_ModemGPIODDR;
     
     /************************************************************************************************/
     /*                                    Defines & Macros Section                                  */
     /************************************************************************************************/
     #define vfnTransceiverRxString               (vfnSPI1ReadString)
     #define vfnTransceiverTxString               (vfnSPI1WriteString)
     #define gau8TransceiverTxBuffer              (gau8SPI1TxBuffer)
     #define gau8TransceiverRxBuffer              (gau8SPI1RxBuffer)
     #define gu8TransceiverTxCounter              (gu8SPI1TxCounter)
     #define SPI_TRANSMITTING                     (gu8SPI1Status &(1<<SPI_TX_IN_PROGRESS))
     #define SPI_RECEIVING                        (gu8SPI1Status &(1<<SPI_RX_IN_PROGRESS))

     #if (BK_TRANSCEIVER_MODULE == BK_DISABLE)
          #define IRQ_INT_VECTOR                       (2)
          
          #define CLK_OUTPUT                           (CLKO_4MHZ)
          #define DESIRED_DBM_VALUE                    (82)            /* -XX dBm to declare Clear Channel */
          #define RF_CHANNEL                           (RF_CHANNEL7)
          #define TX_OUTPUT_POWER                      (0xFF)
     #endif
     
     #define CODE_BYTES_HI                        (0xFF)
     #define CODE_BYTES_LO                        (0x7E)
     
     #define MODEM_IDLE_MODE                      (0x4720)            /* IDLE Mode */
     #define MODEM_CCA_MODE                       (0x4411)            /* CCA mode */
     #define MODEM_ED_MODE                        (0x4721)            /* Energy detect mode */
     #define MODEM_RX_MODE                        (0x4722)            /* Tx, Rx interrupt */
     #define MODEM_TX_MODE                        (0x4723)            /* Tx, Rx interrupt */
     #define MODEM_DOZE_MODE                      (0)
     #define MODEM_HIBER_MODE                     (0)
     
     #define CLKO_16MHZ                           (0x7E80|0x00)
     #define CLKO_8MHZ                            (0x7E80|0x01)
     #define CLKO_4MHZ                            (0x7E80|0x02)
     #define CLKO_2MHZ                            (0x7E80|0x03)
     #define CLKO_1MHZ                            (0x7E80|0x04)
     #define CLKO_62_5KHZ                         (0x7E80|0x05)
     #define CLKO_32_786KHZ                       (0x7E80|0x06)
     #define CLKO_16_393KHZ                       (0x7E80|0x07)

     #define CLEAR_CHN_THRESH                     (UINT8)((DESIRED_DBM_VALUE) *2)
     #define CHANNEL_IDIV                         (0x0F95)
     #define CHANNEL_NUM                          (5)

     #define RF_CHANNEL0                          (UINT32)(0x0F955000)
     #define RF_CHANNEL1                          (UINT32)(0x0F95A000)
     #define RF_CHANNEL2                          (UINT32)(0x0F95F000)
     #define RF_CHANNEL3                          (UINT32)(0x0F964000)
     #define RF_CHANNEL4                          (UINT32)(0x0F969000)
     #define RF_CHANNEL5                          (UINT32)(0x0F96E000)
     #define RF_CHANNEL6                          (UINT32)(0x0F973000)
     #define RF_CHANNEL7                          (UINT32)(0x0F978000)
     #define RF_CHANNEL8                          (UINT32)(0x0F97D000)
     #define RF_CHANNEL9                          (UINT32)(0x0F982000)
     #define RF_CHANNEL10                         (UINT32)(0x0F987000)
     #define RF_CHANNEL11                         (UINT32)(0x0F98C000)
     #define RF_CHANNEL12                         (UINT32)(0x0F991000)
     #define RF_CHANNEL13                         (UINT32)(0x0F996000)
     #define RF_CHANNEL14                         (UINT32)(0x0F99B000)
     #define RF_CHANNEL15                         (UINT32)(0x0F9A0000)
     
     #define GPIO7PIN                             (1<<6)
     #define GPIO6PIN                             (1<<5)
     #define GPIO5PIN                             (1<<4)
     #define GPIO4PIN                             (1<<3)
     #define GPIO3PIN                             (1<<2)
     #define GPIO2PIN                             (1<<1)
     #define GPIO1PIN                             (1<<0)
     
     #define MODEM_TIMER_1MIN                     (UINT32)(15000000)
     #define MODEM_TIMER_10SEG                    (UINT32)(2500000)
     #define MODEM_TIMER_2SEG                     (UINT32)(500000)
     #define MODEM_TIMER_1SEG                     (UINT32)(250000)
     #define MODEM_TIMER_500MSEG                  (UINT32)(75000)
     #define MODEM_TIMER_200MSEG                  (UINT32)(50000)
     #define MODEM_TIMER_150MSEG                  (UINT32)(37500)
     #define MODEM_TIMER_100MSEG                  (UINT32)(25000)
     #define MODEM_TIMER_50MSEG                   (UINT32)(12500)
     #define MODEM_TIMER_10MSEG                   (UINT32)(2500)
     #define MODEM_TIMER_DISABLE                  (UINT16)(0x8000)
     
     /************************************************************************************************/
     /*                                      Hardware Config                                         */
     /************************************************************************************************/
     #if (BK_TRANSCEIVER_MODULE == BK_DISABLE)
          #define RXTXEN_PIN(x)                        (PTFD_PTFD1 = (x))
          #define RXTXEN_DDR(x)                        (PTFDD_PTFDD1 = (x))
          #define ATTN_PIN(x)                          (PTCD_PTCD1 = (x))
          #define ATTN_DDR(x)                          (PTCDD_PTCDD1 = (x))
          #define MODEM_RESET_PIN(x)                   (PTCD_PTCD0 = (x))
          #define MODEM_RESET_DDR(x)                   (PTCDD_PTCDD0 = (x))
     #endif

     /************************************************************************************************/
     /*                                    Extern Variables Section                                  */
     /************************************************************************************************/
     #pragma DATA_SEG DEFAULT
     
     extern _sMachineState sTransceiverMachine;
     extern UINT8 gu8TransceiverStatus;
     extern UINT16 gu16ModemEventsRegister;
     extern INT8 gi8LinkQuality;
     extern INT8 gi8EnDetectValue;     
     extern UINT8 *gpu8RxFramePtr;
     extern UINT8 gu8TransceiverRxByteCounter;
     extern _ModemGPIODDR sGPIODDR;
     extern UINT16 gu16GPIOOut;
     extern UINT8 gu8GPIOIn;

     extern UINT8 gu8TimerCompareReg;
     extern UINT32 gu32TmrCompReloadPeriod;
     extern UINT8 gu8TimeOutFlag;
     /************************************************************************************************/
     /*                                  Function Prototipes Section                                 */
     /************************************************************************************************/
     #pragma CODE_SEG DEFAULT

     /**
     * \brief  Function to configure the data direction register of the transceiver
     * \author Juan Czares Blanco
     * \param  UINT8 GPIO's to be configured
     * \param  UINT8 Pin direction (0 = input, 1= output)
     * \return void
     * \todo   
     * \warning 
     */
     void vfnGPIODir(UINT8 u8GPIO, UINT8 u8Direction);
     
     /**
     * \brief  Function to set High or low the GPIO pins of the transceiver
     * \author Juan Czares Blanco
     * \param  UINT8 GPIO's to be configured
     * \param  UINT8 output level (0 = low, 1= high)
     * \return void
     * \todo   
     * \warning 
     */
     void vfnGPIODataOut(UINT8 u8GPIO, UINT8 u8SetClear);
     
     /**
     * \brief  Function to modify the 7 GPIO pins of the transceiver at the same time
     * \author Juan Czares Blanco
     * \param  UINT8 8-bits Data width, just 7 bits have effect
     * \return void
     * \todo   
     * \warning this function over-write the current output level of the 7 GPIO pins
     */
     void vfnGPIOParallelDataOut(UINT8 u8GPIO);
     
     /**
     * \brief  Function to read the 7 GPIO pins of the transceiver
     * \author Juan Czares Blanco
     * \param  void
     * \return UINT8 input level
     * \todo   
     * \warning the GPIO configured as output will be read as '0'
     */
     UINT8 bfnGPIODataIn(void);
     
     /**
     * \brief  Function to configure a timer base using the transceiver's timers
     * \author Juan Czares Blanco
     * \param  UINT8 timer comparator that will be modified
     * \param  UINT32 this variable has the offset that will be added at the current timer comparator counter
     * \return void
     * \todo   
     * \warning timer comparator counter are 24-bits data width
     */
     void vfnSetModemTimer(UINT8 u8Timer, UINT32 u16Period);
     
     /**
     * \brief  Function to configure the radio, output power, channel, sensibility, etc
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning 
     */
     void vfnRadioInit(void);
     
     /**
     * \brief  Function to Initialize the driver
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning 
     */
     void vfnTransceiverInit(void);
     
     /**
     * \brief  Function to check the current status of the driver
     * \author Juan Czares Blanco
     * \param  void
     * \return UINT8 current status (0 = not busy, 1 = busy)
     * \todo   
     * \warning 
     */
     UINT8 bfnTransceiverBusy(void);
     
     /**
     * \brief  Function to change the operation mode of the transceiver (Tx,Rx,CCA,ED,IDLE)
     * \author Juan Czares Blanco
     * \param  UINT16 operation mode
     * \return void
     * \todo   
     * \warning 
     */
     void vfnSetTransceiverMode(UINT16 u16Mode);
     
     /**
     * \brief  Function to write contiguous registers of the transceiver
     * \author Juan Czares Blanco
     * \param  UINT8 initial address that will be written
     * \param  UINT8* address where the data to be sent are stored
     * \param  UINT8 this parameter has the number of bytes to be written
     * \return void
     * \todo   
     * \warning 
     */
     void vfnTransceiverWriteRegister(UINT8 u8AddressReg, UINT8 *pu8DataBuffer, UINT8 u8DataSize);
     
     /**
     * \brief  Function to write a 16 bit data width register
     * \author Juan Czares Blanco
     * \param  UINT8 Address of register
     * \param  UINT16 16 bit data width
     * \return void
     * \todo   
     * \warning 
     */
     void vfnTransceiverWriteSingleRegister(UINT8 u8AddressReg, UINT16 u16Data);
     
     /**
     * \brief  Function to read a 16 bit data width register
     * \author Juan Czares Blanco
     * \param  UINT8* address where the read data will be stored
     * \param  UINT8 initial address that will be read
     * \param  UINT8 this parameter has the number of bytes to be read
     * \return void
     * \todo   
     * \warning 
     */
     void vfnTransceiverReadRegister(UINT8 *pu8Destiny,UINT8 u8AddressReg, UINT8 u8Counter);
     
     /**
     * \brief  Function to transmit a frame via RF
     * \author Juan Czares Blanco
     * \param  UINT8* address where the data to be written are stored
     * \param  UINT8 this parameter has the number of bytes that will be written
     * \return void
     * \todo   
     * \warning 
     */
     void vfnTransceiverTxMessage(UINT8 *pu8DataBufferPtr, UINT8 u8DataCounter);
     
     /**
     * \brief  Function to manage the transceiver state machine
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning this function must be called on the main loop
     */
     void vfnTransceiverDriver(void);

     /**
     * \brief   Interrupt vector for the IRQ
     * \author  Juan Czares Blanco
     * \param   void
     * \return  void
     * \todo  
     * \warning This function will be allocated in different locations depending the IRQ used; see IRQ_VECTOR definition
     */       
     #pragma CODE_SEG __NEAR_SEG NON_BANKED
     interrupt IRQ_INT_VECTOR void IRQServiceRutine(void);
     /************************************************************************************************/     
#endif
