/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	LinkManagement.h
  \brief    	LinkManagment Configuration header, macros to configure layer interconnections
  \author   	Freescale Semiconductor
  \author      Guadalajara Applications Laboratory RTAC Americas     
  \author   	Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

#ifndef __LINK_MNGM_H__               //Conditional Compilation
     #define __LINK_MNGM_H__

     /************************************************************************************************/
     /*                                      Includes Section                                        */
     /************************************************************************************************/
     #include "derivative.h" /* include peripheral declarations */
     #include "Transceiver.h"
     #include "MyTypes.h"
     
     #include "RadioManagement.h"

     /************************************************************************************************/
     /*                                    Typedef Section                                           */
     /************************************************************************************************/
     typedef enum channel_num_tag
     {
          CHANNEL11,
          CHANNEL12,
          CHANNEL13,
          CHANNEL14,
          CHANNEL15,
          CHANNEL16,
          CHANNEL17,
          CHANNEL18,
          CHANNEL19,
          CHANNEL20,
          CHANNEL21,
          CHANNEL22,
          CHANNEL23,
          CHANNEL24,
          CHANNEL25,
          CHANNEL26
     }channel_num_t;

     /************************************************************************************************/
     /*                                    Defines & Macros Section                                  */
     /************************************************************************************************/
     #define MAX_CHN_NUMBER                       (16)
     #define SAMPLES_PER_CHN                      (1)
     
     #define ENERGY_VALUE                         (gi8EnDetectValue)
     #define ENABLE_TIMEOUT                       (gu8TimeOutFlag = 1)
     #define DISABLE_TIMEOUT                      (gu8TimeOutFlag = 0)
     #define TMR_COMP_REG1                        (gu8TimerCompareReg = TMR_CMP1_A)
     #define TMR_COMP_REG2                        (gu8TimerCompareReg = TMR_CMP3_A)
     #define TMR_COMP_PERIOD                      (gu32TmrCompReloadPeriod)
     #define CHN_LOW_ADDRESS                      (CHANNEL_IDIV)
     #define CHN_HI_ADDRESS                       (CHANNEL_NUM)
     #define CHN_THRESHOLD                        (CLEAR_CHN_THRESH)
     #define GAIN_ADJ_REG                         (CCA_THRESH)
     #define PWR_OUTPUT_REG                       (PA_LVL)
     
     #define TRANSCEIVER_READY                    (!bfnTransceiverBusy())
     #define SET_IDLE_MODE                        (vfnSetTransceiverMode(MODEM_IDLE_MODE))
     #define SET_RX_MODE                          (vfnSetTransceiverMode(MODEM_RX_MODE))
     #define SET_CCA_MODE                         (vfnSetTransceiverMode(MODEM_CCA_MODE))
     #define SET_ED_MODE                          (vfnSetTransceiverMode(MODEM_ED_MODE))
     
     #define WRITE_REGISTERS                      (vfnTransceiverWriteRegister)
     #define WRITE_SINGLE_REG                     (vfnTransceiverWriteSingleRegister)
     #define TRANSMIT_MSG                         (vfnTransceiverTxMessage)

     #define HANDLE_NEW_MSG                       (handle_new_message)
     #define MLMERxDisableRequest(x)              (abort_message(x))  /** Macro to put the transceiver in idle mode */
     #define MLMEGetChannelRequest()              (gu8CurrentRFChannel)

     /************************************************************************************************/
     /*                                    Extern Variables Section                                  */
     /************************************************************************************************/
     #pragma DATA_SEG DEFAULT
     
     extern UINT8 gau8EnergyValuePerChannel[];

     /************************************************************************************************/
     /*                                  Function Prototipes Section                                 */
     /************************************************************************************************/
     #pragma CODE_SEG DEFAULT

     /**
     * \brief  Function to transmit strings via wireless and to set a timeout
     * \author Juan Czares Blanco
     * \param  message_t* address where the data to be sent are stored
     * \return void
     * \todo   
     * \warning 
     */
     void MCPSDataRequest(message_t *LocalMessage);
     
     /**
     * \brief  Function to set the RF channel
     * \author Juan Czares Blanco
     * \param  UINT8 this parameter has the number of the channel to be set
     * \return void
     * \todo   
     * \warning
     */
     void MLMESetChannelRequest(channel_num_t u8Channel);
     
     /**
     * \brief  Function to put the transceiver in reception mode and set a timeout
     * \author Juan Czares Blanco
     * \param  message_t* address where the incoming data will be stored
     * \param  UINT32 this parameter has the time period that is added to the timer comparator counter
     * \return void
     * \todo   
     * \warning 
     */
     void MLMERxEnableRequest(message_t *LocalMessage,UINT32 u32TimeOut);
     
     /**
     * \brief  Function to execute a Clear Channel Assessment operation (CCA)
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning 
     */
     void MLMEClearChannelAssessment(void);
     
     /**
     * \brief  Function to execute a Clear Channel Assessment operation (CCA)
     * \author Juan Czares Blanco
     * \param  void
     * \return void
     * \todo   
     * \warning 
     */
     FuncReturn_t MLMEEnergyDetect(message_t *LocalMessage);
     
     /**
     * \brief  Function to adjust the reception sensibility
     * \author Juan Czares Blanco
     * \param  UINT8 this parameter has the sensibility value in dBm's
     * \return void
     * \todo   
     * \warning 
     */
     void MLMEGainAdjust(UINT8 u8dBmValue);
     
     /**
     * \brief  Function to adjust the transmitter output power
     * \author Juan Czares Blanco
     * \param  UINT8 this parameter has the value of the desired output power
     * \return void
     * \todo   
     * \warning 
     */
     void MLMEPAOutputAdjust(UINT8 u8dBmValue);
     
     /**
     * \brief  Function to execute an energy detect operation over all RF channels
     * \author Juan Czares Blanco
     * \param  UINT16 this parameter has the channel mask to scan the desired channels only
     * \return UINT8 status (0 = in progress, 1 = done)
     * \todo   
     * \warning 
     */
     UINT8 MLMEScanRequest(message_t *LocalMessage);
     
     /**
     * \brief  Function to disable a timer comparator
     * \author Juan Czares Blanco
     * \param  UINT8 timer comparator number to be disabled
     * \return void
     * \todo   
     * \warning 
     */
     void MLMEDisableTimer(UINT8 u8TmrCompare);
     
     /************************************************************************************************/
#endif