/************************************************************************************
* Radio Management implementation
*
* Author(s): Joaquin Garcia (JG)
*
* (c) Copyright 2006, Freescale, Inc.  All rights reserved.
*
* Freescale Confidential Proprietary
*
* No part of this document must be reproduced in any form - including copied,
* transcribed, printed or by any electronic means - without specific written
* permission from Freescale.
*
* Last Inspected:
* Last Tested:
************************************************************************************/

#include "RadioManagement.h"
#include "RadioMngmntWrapper.h"

/************************************************************************************
*************************************************************************************
* Private macros
*************************************************************************************
************************************************************************************/
#define mStartclk_init_c    (0x00000000)
#define mCplclk_init_c      (0x00000000)
#define mSftclk_init_c      (0x00000000)

#define is_any_action_complete_evnt() \
                      ( is_tx_action_completed_evnt() || \
                        is_rx_action_completed_evnt() || \
                        is_action_completed_evnt() )

/************************************************************************************
*************************************************************************************
* Private prototypes
*************************************************************************************
************************************************************************************/
static FuncReturn_t process_tx_msg (void);
static FuncReturn_t process_rx_msg (void);
static FuncReturn_t process_ed_msg (void);
static FuncReturn_t process_to_msg (void);

static bool_t is_a_handled_message(message_t * msg);

/************************************************************************************
*************************************************************************************
* Private type definitions
*************************************************************************************
************************************************************************************/
#define MAX_NUM_MSG (4)
#define PROCESS_TIMEOUT  (255)

#define _inc_msg(x) (((MAX_NUM_MSG-1) == x)? 0: x+1)


#ifdef PROCESS_TO_ENBLE
#define _validate_processs_timeout(x) \
  do { \
    if(gNull_c == mProcess_timeout){ \
         (maMessage_Queu[current_msg])->u8Status.msg_state = x; \
      release_current_message(); \
    } \
    else { \
      /* Do nothing */ \
    } \
  } while(FALSE)
#endif

/************************************************************************************
*************************************************************************************
* Private memory declarations
*************************************************************************************
************************************************************************************/
static uint8_t mRetries_timmer;

#ifdef PROCESS_TO_ENBLE
static uint8_t mProcess_timeout;
#endif

static uint8_t current_msg;
static uint8_t next_msg_to_store;
static message_t* maMessage_Queu[MAX_NUM_MSG];
static uint32_t maMessageTO[MAX_NUM_MSG];

static uint32_t mStartclk_offset = mStartclk_init_c;
static uint32_t mCplclk_offset   = mCplclk_init_c;
static uint32_t mSftclk_offset   = mSftclk_init_c;

#define _set_startclk_offset(x) (mStartclk_offset = x)
#define _set_cplclk_offset(x)   (mCplclk_offset = x)
#define _set_sftclk_offset(x)   (mSftclk_offset = x)


/* This constant must keep the same order than the msg_type_tag enumeration */
static const callback_t cbProcessMsg_c[MAX_MSG_TYPE] = {
          (callback_t)(process_tx_msg),
          (callback_t)(process_rx_msg),
          (callback_t)(process_ed_msg),
          (callback_t)(process_to_msg) };

static const uint8_t mAbortStates[MAX_MSG_TYPE] = {
          MSG_TX_RQST_ABORT,
          MSG_RX_RQST_ABORT,
          MSG_ED_RQST_ABORT,
          MSG_TO_RQST_ABORT };


/************************************************************************************
*************************************************************************************
* Public memory declarations
*************************************************************************************
************************************************************************************/
evnt_flags_t gRadioEvntFlags;

/************************************************************************************
*************************************************************************************
* Public functions
*************************************************************************************
************************************************************************************/

/************************************************************************************
* abort_message
*
* This function aborts a message
*
************************************************************************************/
FuncReturn_t abort_message (message_t * msg){
  if(FALSE == is_a_handled_message(msg)){
    return gFailOutOfRange_c;
  }
  else{
    /* Do Nothing */
  }
  if(maMessage_Queu[current_msg] == msg){
    command_xcvr_abort();
  }
  else{
    /* Do Nothing */
  }
  msg->u8Status.msg_state = (mAbortStates[msg->u8Status.msg_type]);
  return gSuccess_c;
}

/************************************************************************************
* is_a_handled_message
*
* This function check if a given message is placed at the queu
*
************************************************************************************/
static bool_t is_a_handled_message(message_t * msg){
  uint8_t i;
  for(i=0; i<MAX_NUM_MSG; i++){
    if(maMessage_Queu[i] == msg){
      return TRUE;
    }
    else{
      /* Do Nothing */
    }
  }
  return FALSE;
}


/************************************************************************************
* change_current_message_state
*
* This function the current message state to the "mssg_state"
*
************************************************************************************/
void change_current_message_state (any_mssg_state_t mssg_state){
  if(NULL != maMessage_Queu[current_msg]){
    (maMessage_Queu[current_msg])->u8Status.msg_state = (mssg_state.any_type);
  }
  else{
    /* Do Nothing */
  }
}


/************************************************************************************
* change_current_message_state
*
* This function the callback assosiated with the current message
*
************************************************************************************/
void execute_current_message_callback (void){
  if(NULL != maMessage_Queu[current_msg]){
    if(NULL != ((maMessage_Queu[current_msg])->cbDataIndication)){
      (maMessage_Queu[current_msg])->cbDataIndication();
    }
    else {
      /* Do Nothing */
    }
  }
  else{
    /* Do Nothing */
  }
}

/************************************************************************************
* get_current_message
*
* This function returns a pointer to the message that is been processed, it will 
* return NULL if there is no message.
*
************************************************************************************/
message_t* get_current_message (void){
  return (maMessage_Queu[current_msg]);
}


/************************************************************************************
* handle_new_message
*
* This function link a new msg to the queu.
*
************************************************************************************/
FuncReturn_t handle_new_message (message_t * msg, uint32_t timeout){
  if(NULL != maMessage_Queu[next_msg_to_store]){
    return gFailNoResourcesAvailable_c;
  }
  else{
    msg->u8Status.msg_state = initial_state_c;
    maMessage_Queu[next_msg_to_store] = msg;
    maMessageTO[next_msg_to_store]= timeout;
    next_msg_to_store = _inc_msg(next_msg_to_store);
  }
  return gSuccess_c;
}


/************************************************************************************
* release_current_message
*
* This function releases a message from the queu.
*
************************************************************************************/
FuncReturn_t release_current_message (void){
  if(NULL != maMessage_Queu[current_msg]){
    maMessage_Queu[current_msg] = NULL;
    maMessageTO[current_msg] = 0;
    current_msg = _inc_msg(current_msg);
  }
  else{
    /* Do Nothing */
  }

  return gSuccess_c;

}


/************************************************************************************
* radio_manager_init
*
* This initializes all the variables involved in the radio management.
*
************************************************************************************/
FuncReturn_t radio_manager_init (void){
  uint8_t queu_count;
  for(queu_count=0; queu_count<MAX_NUM_MSG; queu_count++){
    maMessage_Queu[queu_count] = NULL;
    maMessageTO[queu_count] = 0;
  }
  current_msg = 0;
  next_msg_to_store = 0;
  clear_all_radio_events();
  return gSuccess_c;
}

/************************************************************************************
* process_radio_msg
*
* This function process any TX/RX/EnergyDetect/Timeout pending msg.
*
************************************************************************************/
FuncReturn_t process_radio_msg (void){
  if(NULL != maMessage_Queu[current_msg]){
    if(MAX_MSG_TYPE > (maMessage_Queu[current_msg])->u8Status.msg_type){
      cbProcessMsg_c[(maMessage_Queu[current_msg])->u8Status.msg_type]();
    }
    else {
      /* Do Nothing */
    }
  }
  else {
    /* Do Nothing */
  }
  return gSuccess_c;
}


/************************************************************************************
*************************************************************************************
* Private functions
*************************************************************************************
************************************************************************************/

/************************************************************************************
* process_tx_msg
*
* This function process any TX pending msg.
*
************************************************************************************/
static FuncReturn_t process_tx_msg (void){
  msg_tx_state_t state;
  state = (msg_tx_state_t)((maMessage_Queu[current_msg])->u8Status.msg_state);
  switch(state)
  {
    case MSG_TX_RQST:
      {
        clear_all_radio_events();
        mRetries_timmer = MAX_TX_RETRIES;

#ifdef PROCESS_TO_ENBLE
        mProcess_timeout = PROCESS_TIMEOUT;
#endif
        command_xcvr_tx();
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_TX_PASSED_TO_DEVICE;
      }
      break;
    case MSG_TX_PASSED_TO_DEVICE:
      {
        if(is_action_started_evnt()){
          clear_action_started_evnt();
          (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_TX_ACTION_STARTED;
        }
        else{
#ifdef PROCESS_TO_ENBLE
          _t_dec(mProcess_timeout);
#endif
        }
      }
      break;
    case MSG_TX_ACTION_STARTED:
      {
        if(is_tx_action_completed_evnt()){
          clear_tx_action_completed_evnt();
          if(is_tx_ac_success_evnt()) {
            clear_tx_ac_success_evnt();
            (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                       MSG_TX_ACTION_COMPLETE_SUCCESS;
            release_current_message();
          }
          else {
            if(gNull_c != mRetries_timmer){
              _t_dec(mRetries_timmer);
              clear_all_radio_events();
              command_xcvr_tx();
              (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                              MSG_TX_PASSED_TO_DEVICE;
            }
            else {
              (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                          MSG_TX_ACTION_COMPLETE_FAIL;
              release_current_message();
            }
          }
        }
        else {
#ifdef PROCESS_TO_ENBLE
          _t_dec(mProcess_timeout);
#endif
        }
      }
      break;
    case MSG_TX_RQST_ABORT:
      {
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_TX_ABORTED;
        release_current_message();
      }
      break;
    case MSG_TX_ABORTED:
    case MSG_TX_ACTION_COMPLETE_SUCCESS:
    case MSG_TX_ACTION_COMPLETE_FAIL:
    default:
      {
        release_current_message();
      }
      break;
  }
#ifdef PROCESS_TO_ENBLE
  _validate_processs_timeout(MSG_TX_ACTION_COMPLETE_FAIL);
#endif
  return gSuccess_c;
}

/************************************************************************************
* process_rx_msg
*
* This function process any RX pending msg.
*
************************************************************************************/
static FuncReturn_t process_rx_msg (void){
  msg_rx_state_t state;
  state = (msg_rx_state_t)((maMessage_Queu[current_msg])->u8Status.msg_state);
  switch(state)
  {
    case MSG_RX_RQST:
      {
        clear_all_radio_events();
        mRetries_timmer = MAX_RX_RETRIES;
#ifdef PROCESS_TO_ENBLE
        mProcess_timeout = PROCESS_TIMEOUT;
#endif
        (maMessage_Queu[current_msg])->u8BufSize = (0x00);
        _set_cplclk_offset(maMessageTO[current_msg]);
        command_xcvr_rx();
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_RX_PASSED_TO_DEVICE;
      }
      break;
    case MSG_RX_PASSED_TO_DEVICE:
      {
        if(is_action_started_evnt()){
          clear_action_started_evnt();
          (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_RX_ACTION_STARTED;
        }
        else{
          /* Do nothing */
        }
      }
      break;
    case MSG_RX_ACTION_STARTED:
      {
        if(is_rx_action_completed_evnt()){
          clear_rx_action_completed_evnt();
          if(is_rx_ac_success_evnt()) {
            clear_rx_ac_success_evnt();
            (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                       MSG_RX_ACTION_COMPLETE_SUCCESS;
            (maMessage_Queu[current_msg])->u8BufSize = (uint8_t)(get_rx_packet_sz());
            release_current_message();
          }
          else {
            if(is_timeout_evnt()){
                clear_timeout_evnt();
                (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_RX_TIMEOUT_FAIL;
                release_current_message();
            }
            else {
            if(gNull_c != mRetries_timmer){
              _t_dec(mRetries_timmer);
                clear_all_radio_events();
                (maMessage_Queu[current_msg])->u8BufSize = (0x00);
              command_xcvr_rx();
                (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                              MSG_RX_PASSED_TO_DEVICE;
            }
            else {
                (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                          MSG_RX_ACTION_COMPLETE_FAIL;
              release_current_message();
            }
          }
        }
        }
        else {
#ifdef PROCESS_TO_ENBLE
          _t_dec(mProcess_timeout);
#endif
        }
      }
      break;
    case MSG_RX_SYNC_FOUND:
      {
        /* Not used at this time */
      }
      break;
    case MSG_RX_RQST_ABORT:
      {
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_RX_ABORTED;
        release_current_message();
      }
      break;
    case MSG_RX_ABORTED:
    case MSG_RX_TIMEOUT_FAIL:
    case MSG_RX_ACTION_COMPLETE_SUCCESS:
    case MSG_RX_ACTION_COMPLETE_FAIL:
    default:
      {
        release_current_message();
      }
      break;
  }
#ifdef PROCESS_TO_ENBLE
  _validate_processs_timeout(MSG_RX_ACTION_COMPLETE_FAIL);
#endif
  return gSuccess_c;
}

/************************************************************************************
* process_ed_msg
*
* This function process any Energy Detect pending msg.
*
************************************************************************************/
static FuncReturn_t process_ed_msg (void){
  static uint32_t prev_chann;
  msg_ed_state_t state;
  state = (msg_ed_state_t)((maMessage_Queu[current_msg])->u8Status.msg_state);
  switch(state)
  {
    case MSG_ED_RQST:
      {
        clear_all_radio_events();
        mRetries_timmer = MAX_ED_RETRIES;
          #ifdef PROCESS_TO_ENBLE
                  mProcess_timeout = PROCESS_TIMEOUT;
          #endif
        /* Use u8BufSize to store the desired channel */
//        if((uintn8_t)MAX_SMAC_CHANNELS < ((maMessage_Queu[current_msg])->u8BufSize)){
        if((uintn8_t)MAX_SMAC_CHANNELS > ((maMessage_Queu[current_msg])->u8BufSize)){
          prev_chann = command_xcvr_ed((maMessage_Queu[current_msg])->u8BufSize);
          (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_ED_PASSED_TO_DEVICE;
        }
        else{
          (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                          MSG_ED_ACTION_COMPLETE_FAIL;
          release_current_message();
        }
      }
      break;
    case MSG_ED_PASSED_TO_DEVICE:
      {
        if(is_action_completed_evnt()){
          clear_action_completed_evnt();
          if(is_ac_success_evnt()) {
            clear_ac_success_evnt();
            (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                       MSG_ED_ACTION_COMPLETE_SUCCESS;          
            *((maMessage_Queu[current_msg])->pu8Buffer->u8Data) = _get_ed_value();
            _set_channel(prev_chann);
            release_current_message();
          }
          else {
            if(gNull_c != mRetries_timmer){
              _t_dec(mRetries_timmer);
              clear_all_radio_events();
              (void)command_xcvr_ed((maMessage_Queu[current_msg])->u8BufSize);
            }
            else {
              (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                          MSG_ED_ACTION_COMPLETE_FAIL;
              _set_channel(prev_chann);
              release_current_message();
            }
          }
        }
        else {
          #ifdef PROCESS_TO_ENBLE
                    _t_dec(mProcess_timeout);
          #endif
        }
      }
      break;
    case MSG_ED_RQST_ABORT:
      {
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_ED_ABORTED;
        release_current_message();
      }
      break;
    case MSG_ED_ABORTED:
    case MSG_ED_ACTION_COMPLETE_SUCCESS:
    case MSG_ED_ACTION_COMPLETE_FAIL:
    default:
      {
        release_current_message();
      }
      break;
  }
#ifdef PROCESS_TO_ENBLE
  _validate_processs_timeout(MSG_ED_ACTION_COMPLETE_FAIL);
#endif
  return gSuccess_c;
}

/************************************************************************************
* process_to_msg
*
* This function process any Timeout pending msg.
*
************************************************************************************/
static FuncReturn_t process_to_msg (void){
  msg_to_state_t state;
  state = (msg_to_state_t)((maMessage_Queu[current_msg])->u8Status.msg_state);
  switch(state)
  {
    case MSG_TO_RQST:
      {
        clear_all_radio_events();
        mRetries_timmer = MAX_TO_RETRIES;
#ifdef PROCESS_TO_ENBLE
        mProcess_timeout = PROCESS_TIMEOUT;
#endif
        command_xcvr_to();
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_TO_PASSED_TO_DIVICE;
      }
      break;
    case MSG_TO_PASSED_TO_DIVICE:
      {
        if(is_action_completed_evnt()){
          clear_action_completed_evnt();
          if(( is_ac_success_evnt()) || 
             ( is_ac_timeout_evnt()) ||
             ( is_timeout_evnt())) {
            clear_ac_success_evnt();
            clear_ac_timeout_evnt();
            clear_timeout_evnt();
            (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                       MSG_TO_ACTION_COMPLETE_SUCCESS;
            release_current_message();
          }
          else {
            if(gNull_c != mRetries_timmer){
              _t_dec(mRetries_timmer);
              clear_all_radio_events();
              command_xcvr_to();
            }
            else {
              (maMessage_Queu[current_msg])->u8Status.msg_state = \
                                                          MSG_TO_ACTION_COMPLETE_FAIL;
              release_current_message();
            }
          }
        }
        else {
#ifdef PROCESS_TO_ENBLE
          _t_dec(mProcess_timeout);
#endif
        }
      }
      break;
    case MSG_TO_RQST_ABORT:
      {
        (maMessage_Queu[current_msg])->u8Status.msg_state = MSG_TO_ABORTED;
        release_current_message();
      }
      break;
    case MSG_TO_ABORTED:
    case MSG_TO_ACTION_COMPLETE_SUCCESS:
    case MSG_TO_ACTION_COMPLETE_FAIL:
    default:
      {
        release_current_message();
      }
      break;
  }
#ifdef PROCESS_TO_ENBLE
  _validate_processs_timeout(MSG_TO_ACTION_COMPLETE_FAIL);
#endif

  return gSuccess_c;
}

/************************************************************************************
*************************************************************************************
* Private Debug stuff
*************************************************************************************
************************************************************************************/


