/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	Transceiver.c
  \brief    	Driver to transmit and receive frames via RF, CCA and ED modes are implemented
  \author   	Freescale Semiconductor
  \author   	Guadalajara Applications Laboratory RTAC Americas
  \author      Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

/************************************************************************************************/
/*                                      Includes Section                                        */
/************************************************************************************************/
#include "Transceiver.h"

/************************************************************************************************/
/*                                     Constants Section                                        */
/************************************************************************************************/
//#pragma CONST_SEG DEFAULT

/************************************************************************************************/
/*                                Function Prototipes Section                                   */
/************************************************************************************************/
//#pragma CODE_SEG DEFAULT

/************************************************************************************************/
/*                                Global Variables Section                                      */
/************************************************************************************************/
//#pragma DATA_SEG DEFAULT




_sMachineState sTransceiverMachine;                    /** Transceiver State Machine Control */
UINT8 gu8TransceiverStatus;                            /** Transceiver Driver Status Register */
UINT8 *gpu8TransDataBufferPtr;                         /** Pointer to fill the transmit buffer with the Payload */
UINT8 *gpu8RxFramePtr = (UINT8 *)(&gau8TransceiverRxBuffer[0]);   /** Pointer to the location where the received
                                                           frame will be stored */
UINT8 gu8TransDataCounter;                             /** Payload Datasize */
UINT16 gu16TransceiverMode;                            /** Variable to change the operation mode */
UINT16 gu16ModemStatus = 0;                            /** Modem Status internal Register */
UINT16 gu16ModemEventsRegister = 0;                    /** Transceiver Status Flags Reister */
UINT8 gu8TransceiverRxByteCounter;                     /** Reception Payload Datasize */
INT8 gi8LinkQuality;                                  /** Received Frame Energy */
INT8 gi8EnDetectValue;                                /** Channel Energy Detect */

UINT16 gu16GPIODDR = 0x007F;                           /** Transceiver Data Direction Register */
UINT16 gu16GPIOOut = 0x0380;                           /** Transceiver Data Output Register */
UINT8 gu8GPIOIn = 0xFF;                                /** Transceiver Data Input Register */
_ModemGPIODDR sGPIODDR;                                /** Struct to configure the pins direction */

UINT8 gu8TimerCompareReg;                              /** Variable to configure the Timer Comparators */
UINT32 gu32TmrCompReloadPeriod;                        /** Time Period of Comparator */

UINT8 gu8TimeOutFlag = 0;                              /** Flag to enable timeout periods */


/**********************************/
message_t Message1;                     /* struct type message (tx,rx) */

UINT8 gau8TxDataBuffer[10];
UINT8 cont_variable;//TEMPORAL

UINT8 gau8TxBuff[3];


/************************************************************************************************/
/*                                      Functions Section                                       */
/************************************************************************************************/
//#pragma CODE_SEG DEFAULT

void vfnGPIODir(UINT8 u8GPIO, UINT8 u8Direction)
{
     UINT16 *pu16Temp = (UINT16*)(&sGPIODDR);
     
     if (u8Direction)
     {
          sGPIODDR.DDRIn &= ~u8GPIO;                                  /* Config pins as output */
     }
     else
     {
          sGPIODDR.DDRIn |= u8GPIO;                                   /* Config pins as input */
     }
     sGPIODDR.DDROut = ~sGPIODDR.DDRIn;
     
     vfnTransceiverWriteSingleRegister(GPIO_DIR,*pu16Temp);
}

void vfnGPIODataOut(UINT8 u8GPIO, UINT8 u8SetClear)
{
     u8GPIO &= 0x7F;
     
     if (u8SetClear)
     {
          gu16GPIOOut |= u8GPIO;                                      /* Set GPIO */
     }
     else
     {
          gu16GPIOOut &= ~u8GPIO;                                     /* Clear GPIO */
     }
     vfnTransceiverWriteSingleRegister(GPIO_DATA_OUT,gu16GPIOOut);
}

void vfnGPIOParallelDataOut(UINT8 u8GPIO)
{
     u8GPIO &= 0x7F;
     gu16GPIOOut &= 0xFF80;
     gu16GPIOOut |= u8GPIO;

     vfnTransceiverWriteSingleRegister(GPIO_DATA_OUT,gu16GPIOOut);
}

UINT8 bfnGPIODataIn(void)
{
     static UINT8 u8Flag = 0;
     
     if (!u8Flag)
     {
          u8Flag = 0x55;
          vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],GPIO_DATA_IN,2);
          return(0);
     }

     u8Flag = 0;
     gu8GPIOIn = gau8TransceiverRxBuffer[1];
     return(1);
}
/************************************************************************************************/

void vfnSetModemTimer(UINT8 u8Timer, UINT32 u16Period)
{
     gu8TimerCompareReg = u8Timer;
     gu32TmrCompReloadPeriod = u16Period;
     
     vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],CURRENT_TIME_A,4);               /* Read Current Timer A,B registers */
     sTransceiverMachine.bNextState = TRANSC_RELOAD_TIMER_STATE0;
}

void vfnTransceiverReloadModemTimerState0(void)
{
     static UINT32 *pu16Ptr;

     pu16Ptr = (UINT32 *)(&gau8TransceiverRxBuffer[1]);
     *pu16Ptr += (UINT32)(gu32TmrCompReloadPeriod);

     gau8TransceiverTxBuffer[0] = (UINT8)(gu8TimerCompareReg &0x3F);
     gau8TransceiverTxBuffer[1] = 0;
     gau8TransceiverTxBuffer[2] = gau8TransceiverRxBuffer[2];
     gau8TransceiverTxBuffer[3] = gau8TransceiverRxBuffer[3];
     gau8TransceiverTxBuffer[4] = gau8TransceiverRxBuffer[4];
     gu8TransceiverTxCounter = 5;
     
     vfnTransceiverTxString(&gau8TransceiverTxBuffer[0],gu8TransceiverTxCounter);
     sTransceiverMachine.bActualState = TRANSC_WAIT_SPI_STATE0;
     sTransceiverMachine.bNextState = TRANSC_IDLE_STATE0;
}

/************************************************************************************************/

void vfnRadioInit(void)
{
     UINT32 u32RFChannel = RF_CHANNEL;
     
     while (bfnTransceiverBusy())                                /* Check Modem On Line */
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],IRQ_STATUS,2);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(REGISTER_11,0x20FF);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(PA_ENABLE,0xFFF7);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(TMR_CMP1_A,0x8000);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }
     
     vfnTransceiverWriteSingleRegister(TMR_CMP2_A,0x8000);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }
     
     vfnTransceiverWriteSingleRegister(TMR_CMP3_A,0x8000);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }
     
     vfnTransceiverWriteSingleRegister(TMR_CMP4_A,0x8000);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(CONTROL_B,0x0E00);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(CCA_THRESH,0x822D);            /* -65 dBm to declare Clear Channel */
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(IRQ_MASK,0x924F);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteRegister(LO1_INT_DIV,(UINT8 *)(&u32RFChannel),4);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(CLKO_CTL,CLK_OUTPUT);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverWriteSingleRegister(PA_LVL,TX_OUTPUT_POWER);                /* MAX POWER */
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }

     vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],RST_IND,2);
     while (bfnTransceiverBusy())
     {
          __RESET_WATCHDOG();
          vfnTransceiverDriver();
     }
}

void vfnTransceiverInit(void)
{
     RXTXEN_PIN(0);
     RXTXEN_DDR(1);                                                   /* Clear RXTXEN */
     ATTN_PIN(1);
     ATTN_DDR(1);                                                     /* Set ATTN */
     MODEM_RESET_PIN(1);
     MODEM_RESET_DDR(1);                                              /* MODEM RESET */
     
//     IRQSC = 0x14;                                                    /* IRQ pin enabled,Interrupt disabled */
     
  /* Enable BDM, Disable Reset */
  SOPT1 = SOPT1_BKGDPE_MASK;
  /* Enable IRQ Pin, Acknowledge Pending IRQs */
  IRQSC = IRQSC_IRQPE_MASK | IRQSC_IRQACK_MASK;

     gu8TransceiverStatus = 0;
     sTransceiverMachine.bActualState = CHECK_TRANSC_ON_LINE;
     sTransceiverMachine.bNextState = TRANSC_IDLE_STATE0;
}

UINT8 bfnTransceiverBusy(void)
{
     if (sTransceiverMachine.bActualState == TRANSC_IDLE_STATE0)
     {
          return(0);
     }
     else
     {
          return(1);
     }
}

void vfnSetTransceiverMode(UINT16 u16Mode)
{
     gu16TransceiverMode = u16Mode;
     sTransceiverMachine.bActualState = TRANSC_CHANGE_MODE_STATE0;
     
     /* Check If TimeOut is Enabled */
     if (gu8TimeOutFlag)
     {
          sTransceiverMachine.bPrevState = TRANSC_SET_TIMEOUT_STATE0;
          return;
     }
     sTransceiverMachine.bPrevState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverWriteRegister(UINT8 u8AddressReg, UINT8 *pu8DataBuffer, UINT8 u8DataSize)
{
     UINT8 u8Index = 0;
     
     gau8TransceiverTxBuffer[u8Index++] = (UINT8)(u8AddressReg &0x3F);
     gu8TransceiverTxCounter = (UINT8)(u8DataSize + 1);
     
     while (u8DataSize--)
     {
          gau8TransceiverTxBuffer[u8Index++] = *pu8DataBuffer++;
     }

     vfnTransceiverTxString(&gau8TransceiverTxBuffer[0],gu8TransceiverTxCounter);
     sTransceiverMachine.bActualState = TRANSC_WAIT_SPI_STATE0;
     sTransceiverMachine.bNextState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverWriteSingleRegister(UINT8 u8AddressReg, UINT16 u16Data)
{
     gau8TransceiverTxBuffer[0] = (UINT8)(u8AddressReg &0x3F);
     gau8TransceiverTxBuffer[1] = (UINT8)(u16Data/0x100);
     gau8TransceiverTxBuffer[2] = (UINT8)(u16Data &0x00FF);
     gu8TransceiverTxCounter = 3;
     
     vfnTransceiverTxString(&gau8TransceiverTxBuffer[0],gu8TransceiverTxCounter);
     sTransceiverMachine.bActualState = TRANSC_WAIT_SPI_STATE0;
     sTransceiverMachine.bNextState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverReadRegister(UINT8 *pu8Destiny,UINT8 u8AddressReg, UINT8 u8Counter)
{
     vfnTransceiverRxString(pu8Destiny,(UINT8)(u8AddressReg|0x80),u8Counter);
     sTransceiverMachine.bActualState = TRANSC_WAIT_SPI_STATE0;
     sTransceiverMachine.bNextState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverTxMessage(UINT8 *pu8DataBufferPtr, UINT8 u8DataCounter)
{
     gpu8TransDataBufferPtr = pu8DataBufferPtr;
     gu8TransDataCounter = u8DataCounter;

     if (u8DataCounter &0x01)                                          /* If DataCounter Odd */
     {
          u8DataCounter += 1;                                          /* Do DataCounter Even */
     }
     
     vfnTransceiverWriteSingleRegister(TX_PKT_CTL,(UINT8)(u8DataCounter +MODEM_HEADER));
     sTransceiverMachine.bNextState = TRANSC_TX_DATA_STATE0;
}

void vfnCheckModemOnLine(void)
{
     if (IRQSC_IRQF)
     {
          IRQSC_IRQACK = 1;
          IRQSC_IRQIE = 1;
          sTransceiverMachine.bActualState = TRANSC_IDLE_STATE0;
     }
}

void vfnTransceiverTxDataState0(void)
{
     UINT8 u8Counter = 0;
     
     gau8TransceiverTxBuffer[u8Counter++] = TX_PKT_RAM;
     
     #ifdef ENABLED_CODE_BYTES
          gau8TransceiverTxBuffer[u8Counter++] = CODE_BYTES_HI;
          gau8TransceiverTxBuffer[u8Counter++] = CODE_BYTES_LO;
     #endif

     #ifdef ENABLE_ENDIAN
          while (gu8TransDataCounter--)                                /* Endian Routine */
          {
               if (u8Counter &0x01)
               {
                    gau8TransceiverTxBuffer[u8Counter++] = *(gpu8TransDataBufferPtr+1);
               }
               else
               {
                    gau8TransceiverTxBuffer[u8Counter++] = *(gpu8TransDataBufferPtr-1);
               }
               gpu8TransDataBufferPtr++;
          }
          
          if (!(u8Counter &0x01))                                      /* DataCounter even */
          {
               gau8TransceiverTxBuffer[u8Counter++] = *(gpu8TransDataBufferPtr-1);
          }
     #else
          while (gu8TransDataCounter--)
          {
               gau8TransceiverTxBuffer[u8Counter++] = *gpu8TransDataBufferPtr++;
          }
          
          if (!(u8Counter &0x01))
          {
               gau8TransceiverTxBuffer[u8Counter++] = 0x00;
          }
     #endif

     vfnTransceiverTxString(&gau8TransceiverTxBuffer[0],u8Counter);
     gu16TransceiverMode = MODEM_TX_MODE;
     sTransceiverMachine.bActualState = TRANSC_WAIT_SPI_STATE0;
     sTransceiverMachine.bNextState = TRANSC_CHANGE_MODE_STATE0;
     sTransceiverMachine.bPrevState = TRANSC_TX_DATA_STATE1;
}

void vfnTransceiverTxDataState1(void)
{
     static UINT16 u16TransmitTimeOut = 2000;
     
     if (gu8TransceiverStatus &(1<<TRANSC_IRQ_PENDING))
     {
          gu8TransceiverStatus &= ~(1<<TRANSC_IRQ_PENDING);
          
          sTransceiverMachine.bActualState = TRANSC_READ_STATUS_STATE0;
          sTransceiverMachine.bNextToNextState = TRANSC_TX_DATA_STATE2;
          
          u16TransmitTimeOut = 2000;
          return;
     }
     
     if (!(u16TransmitTimeOut--))
     {
          u16TransmitTimeOut = 2000;          
          sTransceiverMachine.bErrorState = sTransceiverMachine.bActualState;
          sTransceiverMachine.bActualState = TRANSC_ERROR_STATE0;
     }
}

void vfnTransceiverTxDataState2(void)
{
     if (gu16ModemEventsRegister &(1<<TX_SENT_IRQ))
     {
          gu16ModemEventsRegister &= ~(1<<TX_SENT_IRQ);

          /* Clear Tx Status Flag */
          sTransceiverMachine.bActualState = TRANSC_READ_STATUS_STATE0;

          /* Check If TimeOut is Enabled */
          if (gu8TimeOutFlag)
          {
               #ifdef ALWAYS_RECEIVE
                    gu16TransceiverMode = MODEM_RX_MODE;
                    sTransceiverMachine.bNextToNextState = TRANSC_CHANGE_MODE_STATE0;
                    sTransceiverMachine.bPrevState = TRANSC_SET_TIMEOUT_STATE0;
               #else
                    sTransceiverMachine.bNextToNextState = TRANSC_SET_TIMEOUT_STATE0;
               #endif
          }
          else
          {
               #ifdef ALWAYS_RECEIVE
                    gu16TransceiverMode = MODEM_RX_MODE;
                    sTransceiverMachine.bNextToNextState = TRANSC_CHANGE_MODE_STATE0;
                    sTransceiverMachine.bPrevState = TRANSC_IDLE_STATE0;
               #else
                    sTransceiverMachine.bNextToNextState = TRANSC_IDLE_STATE0;
               #endif
          }
          return;
     }
     sTransceiverMachine.bActualState = TRANSC_TX_DATA_STATE1;
}

void vfnTransceiverGetModemStatusState0(void)
{
     vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],IRQ_STATUS,2);
     sTransceiverMachine.bNextState = TRANSC_READ_STATUS_STATE1;
}

void vfnTransceiverGetModemStatusState1(void)
{
//     UINT16 *pu16BuffPtr = (UINT16 *)(&gau8TransceiverRxBuffer[1]);
//     gu16ModemStatus = (UINT16)(*pu16BuffPtr);

     gu16ModemStatus = gau8TransceiverRxBuffer[1];
     gu16ModemStatus <<= 8;
     gu16ModemStatus |= gau8TransceiverRxBuffer[2];
     
     gu16ModemEventsRegister |= gu16ModemStatus;

     sTransceiverMachine.bActualState = sTransceiverMachine.bNextToNextState;
}

void vfnTransceiverCheckPacketReceivedState(void)
{
     if (gu16ModemStatus &(1<<RX_RCVD_IRQ))
     {
          gu8TransceiverStatus |= (1<<TRANSC_RECEIVED_PCK);
          
          if (gu16ModemStatus &(1<<CRC_VALID))
          {
               gu8TransceiverStatus |= (1<<TRANSC_CRC_VALID);
               
               /* Packet received as valid */
               vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],RX_STATUS,2);
               sTransceiverMachine.bNextState = TRANSC_RECEIVING_STATE0;
               return;
          }
     }

     if (gu16TransceiverMode == MODEM_ED_MODE)
     {
          if (gu16ModemStatus &(1<<CCA_IRQ))
          {
               vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],RX_STATUS,2);
               sTransceiverMachine.bNextState = TRANSC_ENERGY_DET_STATE0;
               return;
          }
     }
     
     if (gu16TransceiverMode == MODEM_CCA_MODE)
     {
          if(gu16ModemStatus &(1<<CCA_IRQ))
          {
               gu16ModemEventsRegister &= ~(1<<CCA_IRQ);
               gu8TransceiverStatus |= (1<<TRANSC_CCA_DONE);
               
               if (!(gu16ModemStatus &(1<<CCA)))
               {
                    //channel not busy
                    gu8TransceiverStatus |= (1<<TRANSC_CHANNEL_CLEAR);
               }
               
               sTransceiverMachine.bActualState = TRANSC_IDLE_STATE0;
               return;
          }
     }

     if (gu16ModemEventsRegister &(1<<PLL_LOCK_IRQ))
     {
          asm("NOP");
     }
          
     #ifdef ALWAYS_RECEIVE
          gu16TransceiverMode = MODEM_RX_MODE;
          sTransceiverMachine.bActualState = TRANSC_CHANGE_MODE_STATE0;
          sTransceiverMachine.bPrevState = TRANSC_IDLE_STATE0;
     #else
          sTransceiverMachine.bActualState = TRANSC_IDLE_STATE0;
     #endif
}

void vfnTransceiverEnergyDetectState0(void)
{
     gi8EnDetectValue = ~(gau8TransceiverRxBuffer[1]);
     gi8EnDetectValue /= 2;
     sTransceiverMachine.bActualState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverReceivingState0(void)
{
     gi8LinkQuality = ~(gau8TransceiverRxBuffer[1]);
     gi8LinkQuality /= 2;
     gu8TransceiverRxByteCounter = (UINT8)(gau8TransceiverRxBuffer[2] -2);        /* without CRC */

     #ifdef ENABLE_ENDIAN
          if (gau8TransceiverRxBuffer[2] &0x01)                            /* DataCounter Odd */
          {
               gau8TransceiverRxBuffer[2] += 1;
          }
     #endif
     
     vfnTransceiverReadRegister(gpu8RxFramePtr,RX_PKT_RAM,gau8TransceiverRxBuffer[2]);
     sTransceiverMachine.bNextState = TRANSC_RECEIVING_STATE1;
}

void vfnTransceiverReceivingState1(void)
{
     #ifdef ENABLE_ENDIAN
          UINT8 u8RxDataLSB;
          UINT8 u8RxCounter;
          
          #ifdef ENABLED_CODE_BYTES
               UINT8 u8Index = 5;
          #else
               UINT8 u8Index = 3;
          #endif
          
          u8RxCounter = (gu8TransceiverRxByteCounter+1);
          u8RxCounter >>= 1;

          while (u8RxCounter--)                                            /* Endian Routine */
          {
               u8RxDataLSB = *(gpu8RxFramePtr+u8Index);
               u8Index++;
               *(gpu8RxFramePtr+(u8Index-1)) = *(gpu8RxFramePtr+u8Index);
               *(gpu8RxFramePtr+u8Index) = u8RxDataLSB;
               u8Index++;
          }
     #endif
     
     gu8TransceiverStatus |= (1<<TRANSC_RX_FRAME_VALID);
     gu8TransceiverStatus &= ~(1<<TRANSC_RECEIVED_PCK);
     gu8TransceiverStatus &= ~(1<<TRANSC_CRC_VALID);
     sTransceiverMachine.bActualState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverChangeModeState0(void)
{
     RXTXEN_PIN(0);                                                        /* Clear RXTXEN */
     sTransceiverMachine.bActualState = TRANSC_CHANGE_MODE_STATE1;
}

void vfnTransceiverChangeModeState1(void)
{
     vfnTransceiverWriteSingleRegister(CONTROL_A,gu16TransceiverMode);
     sTransceiverMachine.bNextState = TRANSC_CHANGE_MODE_STATE2;
}

void vfnTransceiverChangeModeState2(void)
{
     RXTXEN_PIN(1);                                                        /* Set RXTXEN */
     sTransceiverMachine.bActualState = sTransceiverMachine.bPrevState;
}

void vfnTransceiverTimeOutState0(void)
{
     gu8TimeOutFlag = 0;
     
     vfnTransceiverReadRegister(&gau8TransceiverRxBuffer[0],CURRENT_TIME_A,4);                         /* Read Current Timer A,B registers */
     sTransceiverMachine.bNextState = TRANSC_RELOAD_TIMER_STATE0;
}

void vfnWaitSPIReady(void)
{
     if ((!SPI_TRANSMITTING) && (!SPI_RECEIVING))
     {
          sTransceiverMachine.bActualState = sTransceiverMachine.bNextState;
     }
}

void vfnTransceiverErrorState0(void)
{
     sTransceiverMachine.bActualState = TRANSC_READ_STATUS_STATE0;
     sTransceiverMachine.bNextToNextState = TRANSC_IDLE_STATE0;
}

void vfnTransceiverIDLEState(void)
{

}

void (* const paTransceiverFunctions[])(void) =
{
     vfnCheckModemOnLine,
     vfnTransceiverReloadModemTimerState0,
     vfnTransceiverTxDataState0,
     vfnTransceiverTxDataState1,
     vfnTransceiverTxDataState2,
     vfnTransceiverGetModemStatusState0,
     vfnTransceiverGetModemStatusState1,
     vfnTransceiverCheckPacketReceivedState,
     vfnTransceiverEnergyDetectState0,
     vfnTransceiverReceivingState0,
     vfnTransceiverReceivingState1,
     vfnTransceiverChangeModeState0,
     vfnTransceiverChangeModeState1,
     vfnTransceiverChangeModeState2,
     vfnTransceiverTimeOutState0,
     vfnWaitSPIReady,
     vfnTransceiverErrorState0,
     vfnTransceiverIDLEState
};

void vfnTransceiverDriver(void)
{
	paTransceiverFunctions[sTransceiverMachine.bActualState]();
	
	if (!bfnTransceiverBusy())
	{
          if (gu8TransceiverStatus &(1<<TRANSC_IRQ_PENDING))
          {
               gu8TransceiverStatus &= ~(1<<TRANSC_IRQ_PENDING);
               
               sTransceiverMachine.bActualState = TRANSC_READ_STATUS_STATE0;
               sTransceiverMachine.bNextToNextState = TRANSC_CHECK_RX_PACKET_STATE0;
          }
	}
}

//#pragma CODE_SEG __NEAR_SEG NON_BANKED


interrupt VectorNumber_Virq void IRQServiceRutine(void)
{
     IRQSC_IRQACK = 1;
     
     gu8TransceiverStatus |= (1<<TRANSC_IRQ_PENDING);
}

/******************************/

void SMAC_service(void)
{
    UINT8 u8Counter;
    u8Counter=0;
          
          while(!u8Counter)
          {
            
          process_radio_msg();
          
          vfnTransceiverDriver();        
	          
          if(!bfnTransceiverBusy())  
          {                      
                if (tx_msg_action_complete(Message1))
                {                                                                                              
                     u8Counter=1;
                     Message1.u8Status.msg_state = MAX_MSG_TX_STATE;
                     Message1.u8Status.msg_type == MAX_MSG_TYPE;                                         
                } 
                else
                {
                    set_tx_frame_buffer(Message1,gau8TxDataBuffer[0]);
/*
                    tx_msg_payload_buffer(Message1,0) = 0x31;
                    tx_msg_payload_buffer(Message1,1) = cont_variable*3+2; //command
                    tx_msg_payload_buffer(Message1,2) = 0;        
                    tx_msg_payload_buffer(Message1,3) = gau8TxBuff[cont_variable];
*/                  
                    tx_msg_payload_buffer(Message1,0) = gau8TxBuff[0];
                    tx_msg_payload_buffer(Message1,1) = gau8TxBuff[1]; //command
                    tx_msg_payload_buffer(Message1,2) = gau8TxBuff[2];        
                    tx_msg_payload_buffer(Message1,3) = gau8TxBuff[3];
                    
                    if ((cont_variable++)>2) cont_variable=0;                                                    
                    
                    set_tx_datasize(Message1,4+2);
                    set_tx_callback_function(Message1,vfnAnalysePacketReceived);
                    MCPSDataRequest(&Message1);   
                }
          }
          }  
    
}
/**************************************/
void vfnAnalysePacketReceived(void)
{

}





