/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	LinkManagement.c
  \brief    	SMAC Primitives availables for the application layer
  \author   	Freescale Semiconductor
  \author   	Guadalajara Applications Laboratory RTAC Americas
  \author      Juan Czares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

/************************************************************************************************/
/*                                      Includes Section                                        */
/************************************************************************************************/
#include "WirelessLinkMngmnt.h"

/************************************************************************************************/
/*                                     Constants Section                                        */
/************************************************************************************************/
//#pragma CONST_SEG DEFAULT

/************************************************************************************************/
/*                                Function Prototipes Section                                   */
/************************************************************************************************/
//#pragma CODE_SEG DEFAULT


/************************************************************************************************/
/*                                Global Variables Section                                      */
/************************************************************************************************/
//#pragma DATA_SEG DEFAULT

UINT8 gau8EnergyValuePerChannel[MAX_CHN_NUMBER];          /** Array to store Detected Energy per channel */
UINT8 gu8CurrentRFChannel;

/************************************************************************************************/
/*                                      Functions Section                                       */
/************************************************************************************************/
//#pragma CODE_SEG DEFAULT

void MCPSDataRequest(message_t *LocalMessage)
{
     DISABLE_TIMEOUT;
     LocalMessage->u8Status.msg_type = TX;
     LocalMessage->pu8Buffer->reserved[0]=CODE_BYTES_LO;
     LocalMessage->pu8Buffer->reserved[1]=CODE_BYTES_HI;
     HANDLE_NEW_MSG(LocalMessage,0);
}

void MLMESetChannelRequest(channel_num_t u8Channel)
{
     UINT8 au8ChannelOffset[4];
     
     if(u8Channel < MAX_CHN_NUMBER)
     {
          gu8CurrentRFChannel = u8Channel;
          
          au8ChannelOffset[0] = (UINT8)(CHN_LOW_ADDRESS >>8);
          au8ChannelOffset[1] = (UINT8)(CHN_LOW_ADDRESS);
          au8ChannelOffset[1] += (u8Channel/3);
          au8ChannelOffset[2] = (UINT8)((u8Channel +1) * CHN_HI_ADDRESS);
          au8ChannelOffset[2] <<= 4;
          au8ChannelOffset[3] = 0;

          WRITE_REGISTERS(LO1_INT_DIV,&au8ChannelOffset[0],sizeof(au8ChannelOffset));
     }
}

void MLMERxEnableRequest(message_t *LocalMessage,UINT32 u32TimeOut)
{
     if (u32TimeOut)
     {
          ENABLE_TIMEOUT;
          TMR_COMP_REG2;
          TMR_COMP_PERIOD = u32TimeOut;
     }
     else
     {
          DISABLE_TIMEOUT;
     }

     LocalMessage->u8Status.msg_type = RX;
     HANDLE_NEW_MSG(LocalMessage,u32TimeOut);
}

void MLMEClearChannelAssessment(void)
{
     DISABLE_TIMEOUT;
     SET_CCA_MODE;
}

FuncReturn_t MLMEEnergyDetect(message_t *LocalMessage)
{
     LocalMessage->u8Status.msg_type = ED;
     HANDLE_NEW_MSG(LocalMessage,gNull_c);
}

void MLMEGainAdjust(UINT8 u8dBmValue)
{
     UINT16 u16Temp;
     
     u16Temp = (UINT16)(CHN_THRESHOLD <<8);
     u16Temp |= u8dBmValue;

     WRITE_SINGLE_REG(GAIN_ADJ_REG,u16Temp);         
}

void MLMEPAOutputAdjust(UINT8 u8dBmValue)
{
     WRITE_SINGLE_REG(PWR_OUTPUT_REG,u8dBmValue);
}

UINT8 MLMEScanRequest(message_t *LocalMessage)
{
     static UINT8 u8ChannelIndex = 0;
     static UINT8 u8ScanFlag = 0;
     static UINT8 u8SampleCounter = 1;
     static UINT16 u16AverageValue = 0;
     static UINT16 *u16ChanTemp;
     static UINT8 u8PrevChannel;
     
     if (TRANSCEIVER_READY)
     {
          if (!u8ScanFlag)
          {
               u16ChanTemp = (UINT16 *)(&(LocalMessage->pu8Buffer->reserved[0]));
               
               if ((*u16ChanTemp))
               {
                    u8ScanFlag = 3;
                    u16AverageValue = 0;
                    u8ChannelIndex = MLMEGetChannelRequest();
                    u8PrevChannel = MLMEGetChannelRequest();
                    *u16ChanTemp &= ~(1<<u8ChannelIndex);
                    MLMEEnergyDetect(LocalMessage);
                    return(0);
               }
               
               LocalMessage->u8Status.msg_type = MAX_MSG_TYPE;
               return(1);
          }
          
          if (u8ScanFlag == 1)
          {
               while ((*u16ChanTemp))
               {
                    if (u8ChannelIndex == MAX_CHN_NUMBER)
                    {
                         u8ChannelIndex = 0;
                    }
                    
                    if ((*u16ChanTemp) &(1<<u8ChannelIndex))
                    {
                         *u16ChanTemp &= ~(1<<u8ChannelIndex);
                         
                         u8ScanFlag = 2;
                         u16AverageValue = 0;
                         MLMESetChannelRequest(u8ChannelIndex);
                         return(0);
                    }
                    u8ChannelIndex++;
               }
               
               u8ScanFlag = 0;
               MLMESetChannelRequest(u8PrevChannel);
               return(0);
          }
          
          if (u8ScanFlag == 2)
          {
               u8ScanFlag = 3;
               MLMEEnergyDetect(LocalMessage);
               return(0);
          }
          
          if (u8ScanFlag == 3)
          {
               u8ScanFlag = 2;
               u16AverageValue += ENERGY_VALUE;
               
               if ((u8SampleCounter++) == SAMPLES_PER_CHN)
               {
                    u8ScanFlag = 1;
                    u8SampleCounter = 1;
                    gau8EnergyValuePerChannel[u8ChannelIndex] = (u16AverageValue /(UINT16)(SAMPLES_PER_CHN));
                    u8ChannelIndex++;
               }
          }
     }
     return(0);
}

void MLMEDisableTimer(UINT8 u8TmrCompare)
{
     UINT8 au8ClearTimer[] = {0x80,0x00,0x00,0x00};

     WRITE_REGISTERS(u8TmrCompare,&au8ClearTimer[0],4);
}