/*  LICENCE GRANT */
/*
The Software may contain two types of programs: (i) programs enabling you to design a system 
("System Designs"), and (ii) programs that could be executed on your designed system ("System Software"). 
Your rights in these distinct programs differ. With respect to System Designs, Freescale grants to you, 
free of charge, the non-exclusive, non-transferable right to use, reproduce, and prepare derivative works 
of the System Designs for the sole purpose of designing systems that contain a programmable processing 
unit obtained directly or indirectly from Freescale ("Freescale System"). You may not distribute or 
sublicense the System Designs to others; however, you may sell Freescale Systems designed using the System 
Design. Freescale does not grant to you any rights under its patents to make, use, sell, offer to sell, or 
import systems designed using the System Designs. That is beyond the scope of this Agreement. With respect 
to System Software, Freescale grants to you, free of charge, the non-exclusive, non-ttransferable right to 
use, reproduce, prepare derivative works of the System Software, distribute the System Software and 
derivative works thereof in object (machine-readable) form only, and to sublicense to others the right to 
use the distributed System Software exclusively with Freescale Systems. You must prohibit your sublicensees 
from translating, reverse engineering, decompiling, or disassembling the System Software except to the extent 
applicable law specifically prohibits such restriction. If you violate any of the terms or restrictions of 
this Agreement, Freescale may immediately terminate this Agreement, and require that you stop using and 
delete all copies of the Software in your possession or control. You are solely responsible for systems you 
design using the Software. 
*/
#include "MRAM.h"

bool gbMRAMFLAG;

/**************************************************************/
void MRAM_Init(void) 
{
    CNTL = WRITE_ENABLE_MASK | OUTPUTENABLE_MASK | LOWERBYTE_MASK | UPPERBYTE_MASK | CHIP_ENABLE_MASK;

    /* Address data direction as output*/    
    CNTLDD       = OUTPUT; 	  
    ADDRESS_DDFB = OUTPUT;
    ADDRESS_DDSB = OUTPUT;  
    gbMRAMFLAG=false;
}

/**************************************************************/
UINT8 MRAM_ReadByte(void) 
{
    UINT8 Data;                         
    DATADD_HIGH = INPUT;                             
    DATADD_LOW = INPUT;                             
    OUTPUTENABLE  = 0;  
    
    if(gbMRAMFLAG==false)   LOWERBYTE = 0;
    else                    UPPERBYTE = 0;   
    
    CHIP_ENABLE   = 0;                    
    
    asm (nop);   // Delay for MRAM read byte.. 
    
    if(gbMRAMFLAG==false)   Data = DATA_LOW;
    else                    Data = DATA_HIGH;    
    OUTPUTENABLE  = 1;
    LOWERBYTE     = 1;
    UPPERBYTE     = 1;
    CHIP_ENABLE   = 1;
    return Data;
}

/**************************************************************/
void MRAM_WriteByte(UINT8 Data)
{
    if(gbMRAMFLAG==false)   DATA_LOW = Data;
    else                    DATA_HIGH= Data;
    DATADD_LOW = OUTPUT;
    DATADD_HIGH = OUTPUT;
    
    WRITE_ENABLE = 0;
    if(gbMRAMFLAG==false)   LOWERBYTE = 0;
    else                    UPPERBYTE = 0;   
    CHIP_ENABLE  = 0;
    // MRAM takes data in this section
    WRITE_ENABLE = 1;
    LOWERBYTE    = 1;
    UPPERBYTE    = 1;
    CHIP_ENABLE  = 1;
}

/**************************************************************/
void MRAM_SetAddress(UINT32 Address)
{
    _address gu32MRAMAddress;
    gu32MRAMAddress._a = Address;

    ADDRESSLB = gu32MRAMAddress._byte[3];
    ADDRESSMB = gu32MRAMAddress._byte[2];
//    CNTL |= (gu32MRAMAddress._byte[1]&0x03);
    
    A16=gu32MRAMAddress._byte[1]&0x01;
    
    if(gu32MRAMAddress._byte[1] & 0x02)
        A17=1;
    else
        A17=0;


    if(gu32MRAMAddress._byte[1]&0x04)
        gbMRAMFLAG=true;
    else
        gbMRAMFLAG=false;
}
