#ifndef _MENU
#define _MENU

#include "OLED.h"
#include <string.h>
#include "BloodPressure.h"
#include "Storage.h"

#include "RTC.h"
#include "MyTypes.h"
#include "Voice.h"

#define MENU_NUMBERS    7
#define MENU_ITEMS      6
#define MENU_LANGUAGES  4

#define yIconStep  15
#define yMenuStep  15
                     
#define xIconOffset 0
#define yIconOffset 0

#define xMenuOffset 18
#define yMenuOffset 5

#define MAX_MENU_ELEMENTS   4   //not used yet

#define IS_AUDIO_ON()       GET_BIT(AUDIO_ON, MenuFlags)
#define IS_USB_ON()         GET_BIT(USB_ON, MenuFlags)
#define IS_WIRELESS_ON()    GET_BIT(WIRELESS_ON, MenuFlags)
#define IS_SAVE_ALL_ON()    GET_BIT(SAVE_ALL_ON, MenuFlags)
#define ENABLE_SAVE_ALL()   SET_BIT(SAVE_ALL_ON, MenuFlags)

#define AUDIO_ICON      Icons[1][1]
#define USB_ICON        Icons[4][0]
#define WIRELESS_ICON   Icons[4][1]
#define SAVE_ALL_ICON   Icons[3][0]

#define MENU_EXTRAS    6

#define DISPLAY_ERROR_TIME    3000   //in ms


extern UINT8 SelectedIndex;
extern UINT8 MenuIndex;
extern UINT8 MenuFlags;
extern UINT8 Language;
extern UINT8 MenuIndexMin;
extern UINT8 MenuIndexMax;
extern UINT8 IndexOffset;
extern UINT8 MenuUnderFlow;
extern UINT8 PrevMenuIndexStack[3];
extern UINT8 StackCounter;
extern UINT8 IndexStackCounter;
extern UINT8 SelectedIndexStack[3];

extern UINT8 HeartBeatDetected;
extern UINT8 ActualPressure;
extern UINT8 DyastolicPressure;
extern UINT8 SystolicPressure;
extern UINT8 HeartBeat;

extern UINT8 HeartBeatDetectedPrev;

extern UINT8 gu8MenuActualState;
extern UINT8 gu8MenuPrevState;

extern UINT8 KeyHasBeenPressed;


#define PUSH_PREV_INDEX(x)      PrevMenuIndexStack[IndexStackCounter++] = x
#define POP_PREV_INDEX(x)      if(IndexStackCounter) x = PrevMenuIndexStack[--IndexStackCounter] 

#define PUSH_PREV_SELECTED_INDEX(x)      SelectedIndexStack[IndexStackCounter++] = x
#define POP_PREV_SELECTED_INDEX(x)      if(IndexStackCounter) x = SelectedIndexStack[--IndexStackCounter] 

//pos = 1,2 or 3
#define CENTER_OBJ(size, pos)   ((32*pos) - (size/2))


extern UINT8* Icons[MENU_NUMBERS][MENU_ITEMS];
extern const UINT8* MenuItems[MENU_LANGUAGES][MENU_NUMBERS][MENU_ITEMS];

extern const UINT8 MenuSizes[];

extern const UINT8 LanguageIcon[];
extern const UINT8 AudioOffIcon[];
extern const UINT8 AudioOnIcon[];
extern const UINT8 MemoryIcon[];
extern const UINT8 ConnectivityIcon[];
extern const UINT8 TimerIcon[];

extern const UINT8 MexicoIcon[];
extern const UINT8 USAIcon[];
extern const UINT8 FranceIcon[];
extern const UINT8 GermanyIcon[];

extern const UINT8 SaveIcon[];
extern const UINT8 DeleteIcon[];
extern const UINT8 SaveOneIcon[];
extern const UINT8 LoadIcon[];
extern const UINT8 EmptyIcon[];

extern const UINT8 USBOnIcon[];
extern const UINT8 USBOffIcon[]; 
extern const UINT8 WirelessOnIcon[];
extern const UINT8 WirelessOffIcon[];

//main menu
extern const UINT8 Heart[];
extern const UINT8 HeartSmall[];
extern const UINT8 Options[];
extern const UINT8 OptionsSmall[];


// result graphs
extern const UINT8 GraphSystolic[];
extern const UINT8 GraphDyastolic[];
extern const UINT8 GraphHeartBeat[];
extern UINT8* ResultsGraphs[];

extern UINT8 AllSavedValues[3][5];

/*unused*/
extern const UINT8 AdvancedIcon[];

void PaintMenu(void);
void PaintFirstScreen(void);

void vfnClickStart(void);
void vfnClickOptions(void);
 
void vfnClickLanguage(void);
void vfnClickAudio(void);
void vfnClickMemory(void);
void vfnClickConnectivity(void);
void vfnClickPeriodicMeasurements(void);

void vfnClickEnglish(void);
void vfnClickEspanol(void);
void vfnClickFrancoise(void);
void vfnClickDeutsch(void);

void vfnClickSaveAll(void);
void vfnClickClearAll(void);
void vfnClickSaveOne(void);
void vfnClickLoad(void);

void vfnClickUSB(void);
void vfnClickWireless(void);

void vfnClickTimerOff(void);
void vfnClickTimer5min(void);
void vfnClickTimer10min(void);
void vfnClickTimer15min(void);
void vfnClickTimer20min(void);

void vfnClickDummyFunction(void);

void DisplayHeartBeating(void);
void DisplayActualPressure(void);
void DisplayResults(void);
void DisplayGraphResults(UINT8 Selector);

void MenuDisplayError(void);
void MenuDisplaySentOk(void);

extern void (*const ClickAction[MENU_NUMBERS][MENU_ITEMS])(void);

#define AUDIO_ON_MASK   1<<AUDIO_ON
#define SAVED_VALUES        5   //number of saved values

typedef enum
{
    MENU_STATE_FIRST,
    MENU_STATE_ERROR,
    MENU_STATE_RESULTS,
    MENU_STATE_MEASURING,
    MENU_STATE_LOAD_RESULTS,
    MENU_STATE_CLEAR_DATA,
    MENU_STATE_DISPLAY_SENT
    
}_MenuStates;

typedef enum
{
    AUDIO_ON,
    USB_ON,
    WIRELESS_ON,
    SAVE_ALL_ON
} _MenuFlags;
 
typedef enum
{
    FIRST_SCREEN,
    MAIN_MENU,
    LANGUAGE_MENU,
    MEMORY_MENU,
    CONNECTIVITY_MENU,
    PERIODIC_MEASUREMENTS_MENU
} _MenuNames;

typedef enum
{
    ENGLISH,
    ESPANOL,
    FRANCOISE,
    DEUTSCH
}_Languages; 

typedef enum
{
    SYSTOLIC_VALUES, 
    DYASTOLIC_VALUES,
    HEARTBEAT_VALUES  
} _SavedValues;

typedef enum
{
    PRESSURE_STR,
    ERROR_STR,   
    PULSE_STR,
    DELETED_OK_STR,
    SENT_OK_STR     
} _PressureDisplayStrings;


#endif  //_MENU