/**
  Copyright (c) 2006 Freescale Semiconductor
  Freescale Confidential Proprietary
  \file     	SystemConfig.h
  \brief    	System Configuration header, allows to configure the application via Beekit
  \author   	Freescale Semiconductor
  \author      Guadalajara Applications Laboratory RTAC Americas     
  \author   	Juan C?zares Blanco
  \version     1.0
  \date     	02/23/07
  \warning     

  * History:
  
*/

#ifndef __SYSTEM_CONFIG_H__
     #define __SYSTEM_CONFIG_H__

     /************************************************************************************************/
     /*                                      Includes Section                                        */
     /************************************************************************************************/
//     #include "derivative.h" /* include peripheral declarations */
     #include "MyTypes.h"

     /************************************************************************************************/
     /*                                    Defines & Macros Section                                  */
     /************************************************************************************************/

     #define TARGET_BOARD          0
     #define LCD_ENABLED           0

     #define NVTrimRegisterAddr              (0xFFAF)                 //Trim value in non volatile memory
     #define BusFreqOffset                   (BusFreq_10Mhz)


     #define BusFreq_10Mhz                   (0x46)                   //use a subtract operation
     #define BusFreq_9Mhz                    (0x27)                   //use a subtract operation

     #define BusFreq_8Mhz                    (0x00)                   //use an add operation
     #define BusFreq_7Mhz                    (0x31)                   //use an add operation


     #define BK_DISABLE            (0)
     #define BK_ENABLE             (1)
     #define MC9S08QE128           (0)
     #define MC9S08GT60            (1)
     #define MC9S08QD4             (2)
     #define DEVICE                MC9S08QE128

     #if (DEVICE == MC9S08QE128)
          
          /*******************     SCI       *********************************/
          #define BK_SCI1_MODULE                            BK_DISABLE
          
          #if (BK_SCI1_MODULE == BK_ENABLE)     
               #define BK_SCI_PORT                          1
               #define BK_SCI_BAUDRATE                      38400

               #if (BK_SCI_PORT == 1)
                    #define SCI1_BAUDRATE_HI                (SCI1BDH)
                    #define SCI1_BAUDRATE_LO                (SCI1BDL)
                    #define SCI1_DATA_REG                   (SCI1D)
                    #define SCI1_CTR1                       (SCI1C1)
                    #define SCI1_TXEN                       (SCI1C2_TE)
                    #define SCI1_TX_COMPLETE_IEN            (SCI1C2_TCIE)
                    #define SCI1_RXEN                       (SCI1C2_RE)
                    #define SCI1_RXIEN                      (SCI1C2_RIE)
                    #define SCI1_STATUS1                    (SCI1S1)
                    #define SCI1_TX_INT_VECTOR              (16)
                    #define SCI1_RX_INT_VECTOR              (15)
               #elif (BK_SCI_PORT == 2)
                    #define SCI1_BAUDRATE_HI                (SCI2BDH)
                    #define SCI1_BAUDRATE_LO                (SCI2BDL)
                    #define SCI1_DATA_REG                   (SCI2D)
                    #define SCI1_CTR1                       (SCI2C1)
                    #define SCI1_TXEN                       (SCI2C2_TE)
                    #define SCI1_TX_COMPLETE_IEN            (SCI2C2_TCIE)
                    #define SCI1_RXEN                       (SCI2C2_RE)
                    #define SCI1_RXIEN                      (SCI2C2_RIE)
                    #define SCI1_STATUS1                    (SCI2S1)
                    #define SCI1_TX_INT_VECTOR              (23)
                    #define SCI1_RX_INT_VECTOR              (22)
               #else
                    #error "You need to define a Valid SCI channel to use with the Driver"
               #endif

               #define BUS_CLK                              (UINT32)(20000000)
               #define BAUDRATE                             ((UINT16)((UINT32)((UINT32)(BUS_CLK)/(UINT32)(16))/(UINT32)(BK_SCI_BAUDRATE)))
               
               #define SCISOF                               (0x02)
               #define SCIEOF                               (0x03)
               #define SCI_TX_BUFFSIZE                      (40)
               #define SCI_RX_BUFFSIZE                      (40)
          #endif
          /****************************************************/
          
          /*******************     SPI       *********************************/
          #define BK_SPI1_MODULE                            BK_DISABLE
          
          #if (BK_SPI1_MODULE == BK_ENABLE)
               #define BK_SPI_PORT                          1
               
               #if (BK_SPI_PORT == 0)
                    #define SPI1_BAUD_REG                   (SPI0BR)
                    #define SPI1_CTR1                       (SPI0C1)
                    #define SPI1_IEN                        (SPI0C1_SPIE)
                    #define SPI1_CTR2                       (SPI0C2)
                    #define SPI1_DATA_REG                   (SPI0D)
                    #define SPI1_STATUS1                    (SPI0S)
                    #define SPI1_ST1_RX_FLAG                (SPI0S_SPRF)
                    #define SPI1_ST1_TX_FLAG                (SPI0S_SPTEF)
                    #define SPI1_INT_VECTOR                 (15)
                    #define SPI1_RX_BUFF_SIZE               (100)
                    #define SPI1_TX_BUFF_SIZE               (100)
               #elif (BK_SPI_PORT == 1)
                    #define SPI1_BAUD_REG                   (SPI1BR)
                    #define SPI1_CTR1                       (SPI1C1)
                    #define SPI1_IEN                        (SPI1C1_SPIE)
                    #define SPI1_CTR2                       (SPI1C2)
                    #define SPI1_DATA_REG                   (SPI1D)
                    #define SPI1_STATUS1                    (SPI1S)
                    #define SPI1_ST1_RX_FLAG                (SPI1S_SPRF)
                    #define SPI1_ST1_TX_FLAG                (SPI1S_SPTEF)
                    #define SPI1_INT_VECTOR                 (13)
                    #define SPI1_RX_BUFF_SIZE               (100)
                    #define SPI1_TX_BUFF_SIZE               (100)
               #elif (BK_SPI_PORT == 2)
                    #define SPI1_BAUD_REG                   (SPI2BR)
                    #define SPI1_CTR1                       (SPI2C1)
                    #define SPI1_IEN                        (SPI2C1_SPIE)
                    #define SPI1_CTR2                       (SPI2C2)
                    #define SPI1_DATA_REG                   (SPI2D)
                    #define SPI1_STATUS1                    (SPI2S)
                    #define SPI1_ST1_RX_FLAG                (SPI2S_SPRF)
                    #define SPI1_ST1_TX_FLAG                (SPI2S_SPTEF)
                    #define SPI1_INT_VECTOR                 (12)
                    #define SPI1_RX_BUFF_SIZE               (100)
                    #define SPI1_TX_BUFF_SIZE               (100)
               #else
                    #error "You need to define a Valid SPI channel to use with the Driver"
               #endif
          #endif
          /****************************************************/

          /*******************     Transceiver       *********************************/
          #define BK_TRANSCEIVER_MODULE                     BK_ENABLE
          
          #if (BK_TRANSCEIVER_MODULE == BK_ENABLE)
               #define BIG_ENDIAN                           (0)
               #define LITTLE_ENDIAN                        (1)
               #define BK_EN_ENDIAN                         LITTLE_ENDIAN
               #define BK_ALWAYS_RECEIVE                    BK_DISABLE
               #define BK_EN_CODE_BYTES                     BK_DISABLE
               #define BK_CLK_OUTPUT                        CLKO_4MHZ
               #define BK_DESIRED_DBM_VALUE                 65    /* -XX dBm to declare Clear Channel */
               #define BK_RF_CHANNEL                        RF_CHANNEL12
               #define BK_TX_OUTPUT_POWER                   0xFF
               
               #define CLK_OUTPUT                           BK_CLK_OUTPUT
               #define DESIRED_DBM_VALUE                    BK_DESIRED_DBM_VALUE
               #define RF_CHANNEL                           BK_RF_CHANNEL
               #define TX_OUTPUT_POWER                      BK_TX_OUTPUT_POWER
               
               #define IRQ_INT_VECTOR                       (2)
               
               #define RXTXEN_PIN(x)                        (PTBD_PTBD6 = (x))   //Will be PTBD_PTBD6
               #define RXTXEN_DDR(x)                        (PTBDD_PTBDD6 = (x)) //Will be PTBDD_PTBDD6
               #define ATTN_PIN(x)                          (PTBD_PTBD7 = (x))   //Will be PTBD_PTBD7
               #define ATTN_DDR(x)                          (PTBDD_PTBDD7 = (x)) //Will be PTBDD_PTBDD7
               #define MODEM_RESET_PIN(x)                   (PTCD_PTCD4 = (x))   //Will be PTCD_PTCD4
               #define MODEM_RESET_DDR(x)                   (PTCDD_PTCDD4 = (x)) //Will be PTCDD_PTCDD4         
               #define SPI1_SS_PIN(x)                       (PTBD_PTBD5 = (x))
               #define SPI1_SS_DDR(x)                       (PTBDD_PTBDD5 = (x))
               
               #if (BK_ALWAYS_RECEIVE == BK_ENABLE)
                    #define ALWAYS_RECEIVE
               #else
                    #undef ALWAYS_RECEIVE
               #endif
               
               #if (BK_EN_CODE_BYTES == BK_ENABLE)
                    #define ENABLED_CODE_BYTES
               #else
                    #undef ENABLED_CODE_BYTES
               #endif

               #if (BK_EN_ENDIAN == LITTLE_ENDIAN)
                    #define ENABLE_ENDIAN                                          /* Swap payload */
               #else
                    #undef ENABLE_ENDIAN
               #endif
          #endif
          /****************************************************/          
     #elif (DEVICE == MC9S08GT60)
          #include <MC9S08GT60.h>
          
          /*******************     SCI       *********************************/
          #define BK_SCI1_MODULE                            BK_DISABLE
          
          #if (BK_SCI1_MODULE == BK_ENABLE)     
               #define BK_SCI_PORT                          1
               #define BK_SCI_BAUDRATE                      38400

               #if (BK_SCI_PORT == 1)
                    #define SCI1_BAUDRATE_HI                (SCI1BDH)
                    #define SCI1_BAUDRATE_LO                (SCI1BDL)
                    #define SCI1_DATA_REG                   (SCI1D)
                    #define SCI1_CTR1                       (SCI1C1)
                    #define SCI1_TXEN                       (SCI1C2_TE)
                    #define SCI1_TX_COMPLETE_IEN            (SCI1C2_TCIE)
                    #define SCI1_RXEN                       (SCI1C2_RE)
                    #define SCI1_RXIEN                      (SCI1C2_RIE)
                    #define SCI1_STATUS1                    (SCI1S1)
                    #define SCI1_TX_INT_VECTOR              (18)
                    #define SCI1_RX_INT_VECTOR              (17)
               #elif (BK_SCI_PORT == 2)
                    #define SCI1_BAUDRATE_HI                (SCI2BDH)
                    #define SCI1_BAUDRATE_LO                (SCI2BDL)
                    #define SCI1_DATA_REG                   (SCI2D)
                    #define SCI1_CTR1                       (SCI2C1)
                    #define SCI1_TXEN                       (SCI2C2_TE)
                    #define SCI1_TX_COMPLETE_IEN            (SCI2C2_TCIE)
                    #define SCI1_RXEN                       (SCI2C2_RE)
                    #define SCI1_RXIEN                      (SCI2C2_RIE)
                    #define SCI1_STATUS1                    (SCI2S1)
                    #define SCI1_TX_INT_VECTOR              (21)
                    #define SCI1_RX_INT_VECTOR              (20)
               #else
                    #error "You need to define a Valid SCI channel to use with the Driver"
               #endif

               #define BUS_CLK                              (UINT32)(20000000)
               #define BAUDRATE                             ((UINT16)((UINT32)((UINT32)(BUS_CLK)/(UINT32)(16))/(UINT32)(BK_SCI_BAUDRATE)))
               
               #define SCISOF                               (0x02)
               #define SCIEOF                               (0x03)
               #define SCI_TX_BUFFSIZE                      (40)
               #define SCI_RX_BUFFSIZE                      (40)
          #endif
          /****************************************************/
          
          /*******************     SPI       *********************************/
          #define BK_SPI1_MODULE                            BK_DISABLE
          
          #if (BK_SPI1_MODULE == BK_ENABLE)
               #define BK_SPI_PORT                          1
               
               #if (BK_SPI_PORT == 1)
                    #define SPI1_BAUD_REG                   (SPI1BR)
                    #define SPI1_CTR1                       (SPI1C1)
                    #define SPI1_IEN                        (SPI1C1_SPIE)
                    #define SPI1_CTR2                       (SPI1C2)
                    #define SPI1_DATA_REG                   (SPI1D)
                    #define SPI1_STATUS1                    (SPI1S)
                    #define SPI1_ST1_RX_FLAG                (SPI1S_SPRF)
                    #define SPI1_ST1_TX_FLAG                (SPI1S_SPTEF)
                    #define SPI1_INT_VECTOR                 (15)
                    #define SPI1_RX_BUFF_SIZE               (100)
                    #define SPI1_TX_BUFF_SIZE               (100)
               #else
                    #error "You need to define a Valid SPI channel to use with the Driver"
               #endif
          #endif
          /****************************************************/

          /*******************     Transceiver       *********************************/
          #define BK_TRANSCEIVER_MODULE                     BK_ENABLE
          
          #if (BK_TRANSCEIVER_MODULE == BK_ENABLE)
               #define BIG_ENDIAN                           (0)
               #define LITTLE_ENDIAN                        (1)
               #define BK_EN_ENDIAN                         LITTLE_ENDIAN
               #define BK_ALWAYS_RECEIVE                    BK_DISABLE
               #define BK_EN_CODE_BYTES                     BK_DISABLE
               
               #define BK_CLK_OUTPUT                        CLKO_4MHZ
               #define BK_DESIRED_DBM_VALUE                 65    /* -XX dBm to declare Clear Channel */
               #define BK_RF_CHANNEL                        RF_CHANNEL10
               #define BK_TX_OUTPUT_POWER                   0xFF
               
               #define CLK_OUTPUT                           BK_CLK_OUTPUT
               #define DESIRED_DBM_VALUE                    BK_DESIRED_DBM_VALUE
               #define RF_CHANNEL                           BK_RF_CHANNEL
               #define TX_OUTPUT_POWER                      BK_TX_OUTPUT_POWER
               
               #define IRQ_INT_VECTOR                       (2)
               
               #define RXTXEN_PIN(x)                        (PTCD_PTCD3 = (x))
               #define RXTXEN_DDR(x)                        (PTCDD_PTCDD3 = (x))
               #define ATTN_PIN(x)                          (PTCD_PTCD2 = (x))
               #define ATTN_DDR(x)                          (PTCDD_PTCDD2 = (x))
               #define MODEM_RESET_PIN(x)                   (PTCD_PTCD4 = (x))
               #define MODEM_RESET_DDR(x)                   (PTCDD_PTCDD4 = (x))
               #define SPI1_SS_PIN(x)                       (PTED_PTED2 = (x))
               #define SPI1_SS_DDR(x)                       (PTEDD_PTEDD2 = (x))
               
               #if (BK_ALWAYS_RECEIVE == BK_ENABLE)
                    #define ALWAYS_RECEIVE
               #else
                    #undef ALWAYS_RECEIVE
               #endif
               
               #if (BK_EN_CODE_BYTES == BK_ENABLE)
                    #define ENABLED_CODE_BYTES
               #else
                    #undef ENABLED_CODE_BYTES
               #endif

               #if (BK_EN_ENDIAN == LITTLE_ENDIAN)
                    #define ENABLE_ENDIAN                                          /* Swap payload */
               #else
                    #undef ENABLE_ENDIAN
               #endif
          #endif
          /****************************************************/          
     #elif (DEVICE == MC9S08QD4)
          #include <MC9S08QD4.h>
          
          /*******************     SCI       *********************************/
          #define BK_SCI1_MODULE                            BK_DISABLE
          
          #if (BK_SCI1_MODULE == BK_ENABLE)     
               #define BK_SCI_PORT                          1
               #define BK_SCI_BAUDRATE                      38400

               #if (BK_SCI_PORT == 1)
                    #define SCI1_BAUDRATE_HI                (SCI1BDH)
                    #define SCI1_BAUDRATE_LO                (SCI1BDL)
                    #define SCI1_DATA_REG                   (SCI1D)
                    #define SCI1_CTR1                       (SCI1C1)
                    #define SCI1_TXEN                       (SCI1C2_TE)
                    #define SCI1_TX_COMPLETE_IEN            (SCI1C2_TCIE)
                    #define SCI1_RXEN                       (SCI1C2_RE)
                    #define SCI1_RXIEN                      (SCI1C2_RIE)
                    #define SCI1_STATUS1                    (SCI1S1)
                    #define SCI1_TX_INT_VECTOR              (16)
                    #define SCI1_RX_INT_VECTOR              (15)
               #else
                    #error "You need to define a Valid SCI channel to use with the Driver"
               #endif

               #define BUS_CLK                              (UINT32)(20000000)
               #define BAUDRATE                             ((UINT16)((UINT32)((UINT32)(BUS_CLK)/(UINT32)(16))/(UINT32)(BK_SCI_BAUDRATE)))
               
               #define SCISOF                               (0x02)
               #define SCIEOF                               (0x03)
               #define SCI_TX_BUFFSIZE                      (40)
               #define SCI_RX_BUFFSIZE                      (40)
          #endif
          /****************************************************/
          
          /*******************     SPI       *********************************/
          #define BK_SPI1_MODULE                            BK_DISABLE
          
          #if (BK_SPI1_MODULE == BK_ENABLE)
               #define BK_SPI_PORT                          0
               
               #if (BK_SPI_PORT == 0)
                    #define VMisoPinIO(x)                    (PTADD_PTADD2 = (x))
                    #define VMisoPin                         (PTAD_PTAD2)
                    #define VMosiPinIO(x)                    (PTADD_PTADD3 = (x))
                    #define VMosiPin                         (PTAD_PTAD3)
                    #define VClkPinIO(x)                     (PTADD_PTADD1 = (x))
                    #define VClkPin                          (PTAD_PTAD1)
                    
                    extern unsigned char DummyVar;
                    #define SPI1_BAUD_REG                   (DummyVar)
                    #define SPI1_CTR1                       (DummyVar)
                    #define SPI1_IEN                        (DummyVar)
                    #define SPI1_CTR2                       (DummyVar)
                    #define SPI1_DATA_REG                   (DummyVar)
                    #define SPI1_STATUS1                    (DummyVar)
                    #define SPI1_ST1_RX_FLAG                (DummyVar)
                    #define SPI1_ST1_TX_FLAG                (DummyVar)
                    #define SPI1_INT_VECTOR                 (0)
                    #define SPI1_RX_BUFF_SIZE               (30)
                    #define SPI1_TX_BUFF_SIZE               (30)
               #else
                    #error "You need to define a Valid SPI channel to use with the Driver"
               #endif
          #endif
          /****************************************************/

          /*******************     Transceiver       *********************************/
          #define BK_TRANSCEIVER_MODULE                     BK_ENABLE
          
          #if (BK_TRANSCEIVER_MODULE == BK_ENABLE)
               #define BIG_ENDIAN                           (0)
               #define LITTLE_ENDIAN                        (1)
               #define BK_EN_ENDIAN                         LITTLE_ENDIAN
               #define BK_ALWAYS_RECEIVE                    BK_DISABLE
               #define BK_EN_CODE_BYTES                     BK_DISABLE

               #define BK_CLK_OUTPUT                        CLKO_4MHZ
               #define BK_DESIRED_DBM_VALUE                 65    /* -XX dBm to declare Clear Channel */
               #define BK_RF_CHANNEL                        RF_CHANNEL10
               #define BK_TX_OUTPUT_POWER                   0xFF
               
               #define CLK_OUTPUT                           BK_CLK_OUTPUT
               #define DESIRED_DBM_VALUE                    BK_DESIRED_DBM_VALUE
               #define RF_CHANNEL                           BK_RF_CHANNEL
               #define TX_OUTPUT_POWER                      BK_TX_OUTPUT_POWER
               
               #define IRQ_INT_VECTOR                       (3)
               
               extern unsigned char DummyVar;
               #define RXTXEN_PIN(x)                   (vfnGPIODataOut(GPIO1Pin,0))
               #define RXTXEN_DDR(x)                   (vfnGPIODir(GPIO1Pin,1))
               #define ATTN_PIN(x)                     (DummyVar = (x))
               #define ATTN_DDR(x)                     (DummyVar = (x))
               #define MODEM_RESET_PIN(x)              (DummyVar = (x))
               #define MODEM_RESET_DDR(x)              (DummyVar = (x))
               #define SPI1_SS_PIN(x)                  (PTAD_PTAD0 = (x))
               #define SPI1_SS_DDR(x)                  (PTADD_PTADD0 = (x))
               
               #if (BK_ALWAYS_RECEIVE == BK_ENABLE)
                    #define ALWAYS_RECEIVE
               #else
                    #undef ALWAYS_RECEIVE
               #endif
               
               #if (BK_EN_CODE_BYTES == BK_ENABLE)
                    #define ENABLED_CODE_BYTES
               #else
                    #undef ENABLED_CODE_BYTES
               #endif

               #if (BK_EN_ENDIAN == LITTLE_ENDIAN)
                    #define ENABLE_ENDIAN                                          /* Swap payload */
               #else
                    #undef ENABLE_ENDIAN
               #endif
          #endif
          /****************************************************/          
     #else
          #error "Invalid Derivative"
     #endif

     /************************************************************************************************/
     /*                                    GPIO pins and MCU Registers                               */
     /************************************************************************************************/


     /************************************************************************************************/
#endif
