/*  LICENCE GRANT */
/*
The Software may contain two types of programs: (i) programs enabling you to design a system 
("System Designs"), and (ii) programs that could be executed on your designed system ("System Software"). 
Your rights in these distinct programs differ. With respect to System Designs, Freescale grants to you, 
free of charge, the non-exclusive, non-transferable right to use, reproduce, and prepare derivative works 
of the System Designs for the sole purpose of designing systems that contain a programmable processing 
unit obtained directly or indirectly from Freescale ("Freescale System"). You may not distribute or 
sublicense the System Designs to others; however, you may sell Freescale Systems designed using the System 
Design. Freescale does not grant to you any rights under its patents to make, use, sell, offer to sell, or 
import systems designed using the System Designs. That is beyond the scope of this Agreement. With respect 
to System Software, Freescale grants to you, free of charge, the non-exclusive, non-ttransferable right to 
use, reproduce, prepare derivative works of the System Software, distribute the System Software and 
derivative works thereof in object (machine-readable) form only, and to sublicense to others the right to 
use the distributed System Software exclusively with Freescale Systems. You must prohibit your sublicensees 
from translating, reverse engineering, decompiling, or disassembling the System Software except to the extent 
applicable law specifically prohibits such restriction. If you violate any of the terms or restrictions of 
this Agreement, Freescale may immediately terminate this Agreement, and require that you stop using and 
delete all copies of the Software in your possession or control. You are solely responsible for systems you 
design using the Software. 
*/
#include "Voice.h"


/* Variables */
UINT32 gu32PlaySize;
UINT32 gu32PlayIndex;
bool   gbVoiceBusy;             // Sound file is playing Flag

extern UINT8 gu8SCI2_DATA;
extern bool SCI2_Interrupt;

/*************************************************************************************/
void Voice_Init(void) 
{
    
    gbVoiceBusy=false;      // Initialize VoiceBusy Flag
    /* PWM configuration (TPM3 Channel 0) */ 
    TPM3SC= TPM3SC_CLKSA_MASK|PWM_RESOLUTION;           // Selecciona bus interno,
    TPM3MOD=PWM_MODULO;
    TPM3C0SC=TPM3C0SC_MS0B_MASK;
    /* Time Base configuration */
    TPM1SC= TPM1SC_CLKSA_MASK;           // Selecciona bus interno, 
    TPM1MOD=SAMPLE_RATE; 
}

/*************************************************************************************/
UINT32 VoiceGetAddress(UINT8 u8Number)
{											
    _address Address;
    _address u32TempAddress;
    Address._a=0;
	u32TempAddress._a=0;
	
	u32TempAddress._byte[1]=0;
	u32TempAddress._byte[2]=0;
    u8Number-=1;
    u32TempAddress._byte[3]=u8Number;
    u8Number=u8Number<<1;
    u32TempAddress._a+= u8Number+1;
	Address._byte[0]=0;								
    
    MRAM_SetAddress(u32TempAddress._a); 	
	Address._byte[1]=MRAM_ReadByte();
    MRAM_SetAddress(u32TempAddress._a+1); 	
	Address._byte[2]=MRAM_ReadByte();
    MRAM_SetAddress(u32TempAddress._a+2); 	
	Address._byte[3]=MRAM_ReadByte();
	return(Address._a);					
}

/*************************************************************************************/
void Voice_Play(UINT8 u8PlayIndex)
{
    /* Get Sound file settings */
    _address Address;
    Voice_Stop();
    gu32PlayIndex=VoiceGetAddress(u8PlayIndex);
    Address._byte[0]=0;
    MRAM_SetAddress(gu32PlayIndex+6);
    Address._byte[1]=MRAM_ReadByte();
    MRAM_SetAddress(gu32PlayIndex+5);
    Address._byte[2]=MRAM_ReadByte();
    MRAM_SetAddress(gu32PlayIndex+4);
    Address._byte[3]=MRAM_ReadByte();
    gu32PlaySize=Address._a-120;
    gu32PlayIndex+=120;
    
    gbVoiceBusy=true;       // Set VoiceBusy Flag
    
    /* Enable Timer modules */
    TPM3C0SC_ELS0B=1;    // Enable PWM in chanel 0 TPM3;
    TPM1SC_TOIE=1;       // Enable base time interrupt (125 uS), 

}

/*************************************************************************************/
void Voice_Stop(void)
{
    TPM3C0SC_ELS0B=0;       // Disable PWM
	TPM1SC_TOIE=0;          // Disable time base Interrupt 
    gbVoiceBusy=false;       // Clear VoiceBusy Flag
}

/*************************************************************************************/
void Voice_Loader(void)
{
	_address tempAdd;
	_address tempSize;
	UINT16 u8Index;
	UINT8 u8Sizenumber;
	UINT8 u8Number;
	UINT8 u8Counter;
	UINT32 u8DataSize;
	
	//SCI2_Send_String("\n\rHow Many Sounds?");
  SCI2_Send_byte('a');
    u8Number=0;
    // Get Numer of sounds to load
	while(!SCI2_Interrupt);
	SCI2_Interrupt=false;

	SCI2_Send_byte(gu8SCI2_DATA);
    u8Number=(gu8SCI2_DATA-48)*10;
	while(!SCI2_Interrupt);
	SCI2_Interrupt=false;

	SCI2_Send_byte(gu8SCI2_DATA);
    u8Number+=gu8SCI2_DATA-48;

	MRAM_SetAddress(0);
	MRAM_WriteByte(u8Number);	
	tempAdd._a=(u8Number<<1)+u8Number+1;  				// Calculate Base Memory address (FAT)
  u8Index=1;									   	


  for(u8Counter=1;u8Counter<=u8Number;u8Counter++)
  {
  		tempSize._a=0x0F;
  		
  		// Write index in Memory
  		MRAM_SetAddress(u8Index);
  		MRAM_WriteByte(tempAdd._byte[1]);
  		u8Index++;
  		MRAM_SetAddress(u8Index);
  		MRAM_WriteByte(tempAdd._byte[2]);
  		u8Index++;
  		MRAM_SetAddress(u8Index);
  		MRAM_WriteByte(tempAdd._byte[3]);
  		u8Index++;
  		
  		// 1st byte of the size of sound file
  		SCI2_Send_String((UINT8*)"\n\r\nTranfer Sound ");
  		//Voice_HEXtoChar(u8Counter);
  		
  		SCI2_Interrupt=false;    // Prevent False Interrupt
  		u8DataSize=1;
  	    while(!SCI2_Interrupt);
  	    SCI2_Interrupt=false;
  		MRAM_SetAddress(tempAdd._a);
  		MRAM_WriteByte(gu8SCI2_DATA);
  		u8DataSize++;
  		tempAdd._a++;			
          u8Sizenumber=3;
  		
  		do
  		{
  		    MRAM_SetAddress(tempAdd._a);
  	        while(!SCI2_Interrupt);
  	        SCI2_Interrupt=false;
  			MRAM_WriteByte(gu8SCI2_DATA);
  			if(u8DataSize>4 && u8DataSize<8)					// Save Size of Sound File
  			{
  				tempSize._byte[u8Sizenumber]=gu8SCI2_DATA;		
  				u8Sizenumber--;
  			}
  			tempAdd._a++;			
  			u8DataSize++;
  			
  		}while(u8DataSize <= tempSize._a);
  		SCI2_Send_byte(97);
  };
	SCI2_Send_String((UINT8*)"\r\nLoad OK!");
	
}

/*************************************************************************************/
void interrupt VectorNumber_Vtpm1ovf TPM2OVF_Isr(void)
{
    (void)TPM1SC;
    TPM1SC_TOF=0;
    TPM3C0V=MRAM_ReadByte();
    MRAM_SetAddress(gu32PlayIndex++);
    gu32PlaySize--;									// Decrement size counter
	if(!gu32PlaySize)
			Voice_Stop();
}