/*  LICENCE GRANT */
/*
The Software may contain two types of programs: (i) programs enabling you to design a system 
("System Designs"), and (ii) programs that could be executed on your designed system ("System Software"). 
Your rights in these distinct programs differ. With respect to System Designs, Freescale grants to you, 
free of charge, the non-exclusive, non-transferable right to use, reproduce, and prepare derivative works 
of the System Designs for the sole purpose of designing systems that contain a programmable processing 
unit obtained directly or indirectly from Freescale ("Freescale System"). You may not distribute or 
sublicense the System Designs to others; however, you may sell Freescale Systems designed using the System 
Design. Freescale does not grant to you any rights under its patents to make, use, sell, offer to sell, or 
import systems designed using the System Designs. That is beyond the scope of this Agreement. With respect 
to System Software, Freescale grants to you, free of charge, the non-exclusive, non-ttransferable right to 
use, reproduce, prepare derivative works of the System Software, distribute the System Software and 
derivative works thereof in object (machine-readable) form only, and to sublicense to others the right to 
use the distributed System Software exclusively with Freescale Systems. You must prohibit your sublicensees 
from translating, reverse engineering, decompiling, or disassembling the System Software except to the extent 
applicable law specifically prohibits such restriction. If you violate any of the terms or restrictions of 
this Agreement, Freescale may immediately terminate this Agreement, and require that you stop using and 
delete all copies of the Software in your possession or control. You are solely responsible for systems you 
design using the Software. 
*/

// Includes
#include <hidef.h>                                          // for EnableInterrupts macro 
#include <string.h>
#include "derivative.h"                                     // include peripheral declarations
#include "main.h"                                           // DEMO board specifics
#include "Images.h"
#include "OLED.h"
#include "Menu.h"
#include "EFIELD.h"
#include "RTC.h"
#include "MyTypes.h"
#include "ADC.h"
#include "MCU.h"
#include "Language_Handler.h"


/* Includes Pepe's Project */ 
#include "Voice.h"
#include "MRAM.h"
#include "SCI.h"
#include "Storage.h"

/* Includes Fleik's project */ 
#include "TPM.h"
#include "BloodPressure.h"
#include "tables.h"                

/* Includes SMAC's project */ 
#include "WirelessLinkMngmnt.h"
#include "Transceiver.h"
#include "SPI.h"
#include "MyTypes.h"
#include "RadioManagement.h"
#include "RadioMngmntWrapper.h"





// prototypes


void EFieldStateMachine();
/*
void vfnAnalysePacketReceived(void);
void vfnSPIxcvrInit(void);     
void vfnRadioMng(void);
void SMAC_service(UINT16 SYSPR, UINT16 DIASPR, UINT16 PULSE);
*/
  
void SMAC_Constructor_BloodPresure(UINT8,UINT8);
void EFieldStateMachine(void);

/* Menu Sates */
void vfnMenuStateFirst(void);
void vfnMenuStateError(void);
void vfnMenuStateResults(void);
void vfnMenuStateMeasuring(void);
void vfnMenuLoadResults(void);
void vfnMenuClearData(void);
void vfnMenuSendData(void);

void (*const MenuStateMachine[])(void) = 
{
    vfnMenuStateFirst,
    vfnMenuStateError,
    vfnMenuStateResults,
    vfnMenuStateMeasuring,
    vfnMenuLoadResults,
    vfnMenuClearData,
    vfnMenuSendData
};

#define SYS 0
#define DIA 1
#define PUL 2



typedef enum
{
    KEY_LEFT,
    KEY_RIGHT,
    KEY_UP,
    KEY_DOWN,
    KEY_ENTER
}_KeyValues;


UINT8 KeyPressed;
UINT8 KeyHasBeenPressed;


//variables

//message_t Message1;                     /** message type struct used to transmit and receive */

//UINT8 gau8TxDataBuffer[10];
//UINT8 gu8tx1Rqst,gu8tx2Rqst,gu8tx3Rqst,gu8txcomplete,gu8txdone,gu8out;
volatile extern UINT16 gu16DisplayDelay;
extern bool gbVoiceBusy;
extern bool SCI2_Interrupt; 

extern UINT8 gau8TxBuff[3];


//UINT16 size_t strlen(const char *s);



void main(void) {
  UINT8 n;
  UINT8 u8Counter;

  

  MCU_Init();



  
  ADC_Init();
  EFIELD_Init();
  
  SCI2_Init();
  MRAM_Init();
  Voice_Init();  
  
  Voice_Init();                                     
  TPM2_Init();
  
  /* SMAC SCPI Initalization */
  MODEM_RESET_PIN(0);
  MODEM_RESET_DDR(1);                /* MODEM RESET */
  
  vfnSPI1Init();
  
  EnableInterrupts; 

    /*
    vfnTransceiverInit();
    vfnRadioInit();
    (void) radio_manager_init();

    for(;;)
    {
        
    gau8TxBuff[0]='J';
    gau8TxBuff[1]='O';
    gau8TxBuff[2]='T';
    gau8TxBuff[3]='O';
    SMAC_service();
    }

  */
  ENABLE_SAVE_ALL();
  OLED_StartUpSequence();
  
  Voice_Play(88);       //Play Intro Sound
  
  for(n=127;n>0;n--)  
  {
    OLED_DrawImage(n,7, 0, (UINT8*)LogoFSL);
    Delay(3);                                // delay is not calibrate against amount of data to scroll
  }  
  Delay(2000);      
  for(n=7;n<79;n++)  {
    OLED_DrawImage(0,n, 0, (UINT8*)LogoFSL);
    Delay(3);                                // delay is not calibrate against amount of data to scroll
  }
  Delay(500);      
  
  
  OLED_Fill(0,0, 128,64, 0x00);                           // fill black


  RTC_Init();
  
  pfBP = &vfnBloodPressureIdle;
  PaintMenu();
  // ------------------------
  // main loop
  // ------------------------
  
  
  for EVER 
  {
      EFieldStateMachine();
      pfBP();
      MenuStateMachine[gu8MenuActualState]();       
      
      

  }
}


/*************************************************************************
 *
 *  E-Field Function that updates the KeyPressed and KeyHasBeenPressed
  *
 **************************************************************************/

void EFieldStateMachine()
{
    if(EFIELDStatus == 0 && EFieldReleased == TRUE)
    {
        EFieldReleased = FALSE;
    }

    else if ( (EFIELDStatus> 0) && EFieldReleased == FALSE) //if some electrode is pressed
    {
        Electrodes = EFIELDStatus;
        //SET_BIT(ELECTRODE_PRESSED,Electrodes);      //an electrode was pressed    
        
        KeyHasBeenPressed = TRUE;
        
        if (GET_BIT(ELECTRODE_RIGHT,Electrodes) == 1)
        {
            KeyPressed = KEY_RIGHT;
        }
        else if (GET_BIT(ELECTRODE_LEFT, Electrodes) == 1)
        {
            KeyPressed = KEY_LEFT;  
        }
        else if (GET_BIT(ELECTRODE_UP,Electrodes) == 1)
        {
            KeyPressed = KEY_UP;  
        }
        else if (GET_BIT(ELECTRODE_DOWN,Electrodes) == 1)
        {
            KeyPressed = KEY_DOWN;  
        }
        else if (GET_BIT(ELECTRODE_ENTER,Electrodes) == 1)
        {
            KeyPressed = KEY_ENTER;  
        }
        
        EFieldReleased = TRUE;
        
    }
  
}






/*************************************************************************
 *
 *  Menu States
 *
 *
 **************************************************************************/

void vfnMenuStateFirst(void)
{

    if ( KeyHasBeenPressed) //if some electrode is pressed
    {
         //Electrode Actions for the first screen        
        if  (MenuIndex == 0)
        {
            if (KeyPressed == KEY_RIGHT)
            {
                SelectedIndex = (UINT8) ((SelectedIndex + 1) % MenuSizes[MenuIndex] );
                OLED_CLEAR_SCREEN();
                PaintMenu();
            }

            else if(KeyPressed == KEY_LEFT)
            {
                if (SelectedIndex == 0)
                {
                    MenuUnderFlow = TRUE;
                    SelectedIndex = MenuSizes[MenuIndex] - 1;
                }
                else
                {
                    SelectedIndex--;    
                }
                OLED_CLEAR_SCREEN();
                PaintMenu();
            }

            else if (KeyPressed == KEY_ENTER)
            {
                ClickAction[MenuIndex][SelectedIndex]();

                /*
                if (gu8MenuActualState != MENU_STATE_FIRST)
                {
                    return;
                }*/
            }            
        }
        
        //electrode actions for all other menus
        else
        {
            if ((KeyPressed == KEY_ENTER)) //||(KeyPressed == KEY_RIGHT ) 
            {
                ClickAction[MenuIndex][SelectedIndex]();            

                /*
                if (gu8MenuActualState != MENU_STATE_FIRST)
                {
                    return;
                } */
            }

            else if(KeyPressed == KEY_LEFT)
            {
                if(MenuIndex > 0)
                {
                    POP_PREV_INDEX(MenuIndex); //go to prev menu
                    POP_PREV_SELECTED_INDEX(SelectedIndex); //with selected index

                    OLED_CLEAR_SCREEN();
                    PaintMenu();
                }
            }

            else if (KeyPressed == KEY_DOWN)
            {
                SelectedIndex = (UINT8) ((SelectedIndex + 1) % MenuSizes[MenuIndex] );
                OLED_CLEAR_SCREEN();
                PaintMenu();

            }

            else if (KeyPressed == KEY_UP)
            {
                if (SelectedIndex == 0)
                {
                    MenuUnderFlow = TRUE;
                    SelectedIndex = MenuSizes[MenuIndex] - 1;
                }
                else
                {
                    SelectedIndex--;    
                }            
                OLED_CLEAR_SCREEN();
                PaintMenu();

            }            
        }

        KeyHasBeenPressed = FALSE;

    }
    
     // If GUI (PC) call for data dump
    else if(SCI2_Interrupt==true && IS_USB_ON()==TRUE)
    {
        gu8MenuPrevState = gu8MenuActualState;
        gu8MenuActualState = MENU_STATE_DISPLAY_SENT;
    }


}


void vfnMenuSendData(void)
{
    UINT8 static StoredState;
    
    if(gu8MenuPrevState != MENU_STATE_DISPLAY_SENT)
    {
        StoredState = gu8MenuPrevState;     //save prev state to be able to return to that state
        gu8MenuPrevState = gu8MenuActualState;
        Storage_USB_Manager();    
        SCI2_Interrupt=false;
        MenuDisplaySentOk();
        gu16DisplayDelay = 0;        
    }

    else if(gu16DisplayDelay>=2000)
    {
        gu16DisplayDelay = 0;
        OLED_CLEAR_SCREEN();     
        gu8MenuActualState = StoredState; //restore previous state
        if(gu8MenuActualState == MENU_STATE_FIRST)
        {
            PaintMenu();
        }
        
    }    
}


void vfnMenuClearData(void)
{
    UINT8* StringPtr;
    UINT8 StrLen;
        
    StringPtr = (UINT8*)(MenuItems[Language][MENU_EXTRAS][DELETED_OK_STR]);
    StrLen = (UINT8) strlen(StringPtr);

    if (gu8MenuPrevState == MENU_STATE_FIRST)
    {
        Storage_FIFO_ClearAll();
        gu8MenuPrevState = gu8MenuActualState;
        OLED_WriteString5x7(CENTER_OBJ(StrLen*5,2),32-10,0xFF, (UINT8*)MenuItems[Language][MENU_EXTRAS][DELETED_OK_STR]);
        KeyHasBeenPressed = FALSE;
    }
    
    else if ( KeyHasBeenPressed )   
    {
        if (KeyPressed == KEY_ENTER)
        {
            gu8MenuActualState = MENU_STATE_FIRST;
            OLED_CLEAR_SCREEN();
            PaintMenu();
        }
        KeyHasBeenPressed = FALSE;
    }
    
    else if(gu16DisplayDelay>=2000)
    {
        gu16DisplayDelay = 0;
        OLED_CLEAR_SCREEN();     
        gu8MenuActualState = MENU_STATE_FIRST;
        PaintMenu();
    }
}

void vfnMenuStateError(void)
{    
    if(gu16DisplayDelay>=DISPLAY_ERROR_TIME)
    {
        gu16DisplayDelay = 0;
        gu8MenuPrevState = gu8MenuActualState;
        gu8MenuActualState = MENU_STATE_FIRST;
        OLED_CLEAR_SCREEN();
        PaintMenu();
    }
}



void vfnMenuLoadResults(void)
{
    static UINT8 ResultLoadSelection = SYSTOLIC_VALUES;
    
    if (gu8MenuPrevState != MENU_STATE_LOAD_RESULTS)
    {
        gu8MenuPrevState = gu8MenuActualState;
        
        //Load saved data
        Storage_FIFO_Restore();
        DisplayGraphResults(ResultLoadSelection);
    }

    //if pressed key 
    else if ( KeyHasBeenPressed )   
    {
        if ((KeyPressed == KEY_LEFT) || (KeyPressed == KEY_ENTER))
        {
            gu8MenuActualState = MENU_STATE_FIRST;
            OLED_CLEAR_SCREEN();
            PaintMenu();
        }
        
        else 
        {
            if (KeyPressed == KEY_DOWN)
            {
                ResultLoadSelection = (UINT8) ((ResultLoadSelection + 1) % 3);
                DisplayGraphResults(ResultLoadSelection);
            }

            else if (KeyPressed == KEY_UP)
            {
                if (ResultLoadSelection == 0)
                {
                    ResultLoadSelection = 2;
                }
                else
                {
                    ResultLoadSelection--;    
                }  
                DisplayGraphResults(ResultLoadSelection);
            }

        }
        
        KeyHasBeenPressed = FALSE;
    }

     // If GUI (PC) call for data dump
    else if(SCI2_Interrupt==true && IS_USB_ON()==TRUE)
    {
        gu8MenuPrevState = gu8MenuActualState;
        gu8MenuActualState = MENU_STATE_DISPLAY_SENT;
    }
    
    
}


void vfnMenuStateResults(void)
{  
    static UINT8 ResultSelection = 0;
    UINT8 u8Counter;
    
    if (gu8MenuPrevState == MENU_STATE_MEASURING)
    {
        DisplayResults();
        gu8MenuPrevState = gu8MenuActualState;
        

        /* If saving is enabled, save last results 
           Load saved values
        */
        
        if (IS_SAVE_ALL_ON() == TRUE)
        {
            //store last measurement
            Storge_FIFO_Save(SystolicPressure,DyastolicPressure,HeartBeat);
            //Load saved data
            Storage_FIFO_Restore();
        }
        else
        {
            //display save screen
        }
        
                        

        if(IS_AUDIO_ON() == TRUE)
        {
            if (Language == ENGLISH)
            {
                Voice_Play(YOUR_PRESSURE_IS);               
                Voice_English_Handler(SystolicPressure);                
                Voice_Play(OVER);
                Voice_English_Handler(DyastolicPressure);                      
            }


            else if (Language == ESPANOL)
            {
                Voice_Play(SU_PRESION_ES);
                Voice_Spanish_Handler(SystolicPressure);
                Voice_Play(SOBRE);
                Voice_Spanish_Handler(DyastolicPressure);          
            }
            
            else if(Language == FRANCOISE)
            {
                Voice_Play(VOTRE_PRESSIONEST);
                Voice_French_Handler(SystolicPressure);
                Voice_Play(SUR);
                Voice_French_Handler(DyastolicPressure);          
            }
        }
        
        
        // If Wireles ON send one Measurement
        if(IS_WIRELESS_ON() == TRUE)
        {
            // SMAC Init 
            vfnTransceiverInit();
            vfnRadioInit();
            (void) radio_manager_init();
            
            // Radio already on
            for(u8Counter=0;u8Counter<=20;u8Counter++)
            {
                
                SMAC_Constructor_BloodPresure(SYS,SystolicPressure);
                SMAC_service();                
                SMAC_Constructor_BloodPresure(DIA,DyastolicPressure);
                SMAC_service();                
                SMAC_Constructor_BloodPresure(PUL,HeartBeat);
                SMAC_service();                
            }

            MODEM_RESET_PIN(0);
            IRQSC_IRQIE = 0;

        }
        

    }

    //if pressed key 
    if ( KeyHasBeenPressed )   
    {
        if ((KeyPressed == KEY_LEFT) || (KeyPressed == KEY_ENTER))
        {
            gu8MenuPrevState = gu8MenuActualState;
            gu8MenuActualState = MENU_STATE_FIRST;
                        
            OLED_CLEAR_SCREEN();
            PaintMenu();
        }
        
        else if (IS_SAVE_ALL_ON() == TRUE) 
        {

            if (KeyPressed == KEY_DOWN)
            {
                ResultSelection = (UINT8) ((ResultSelection + 1) % 4);
            }

            else if (KeyPressed == KEY_UP)
            {
                if (ResultSelection == 0)
                {
                    ResultSelection = 3;
                }
                else
                {
                    ResultSelection--;    
                }  
            }


            if ((KeyPressed == KEY_UP)||(KeyPressed == KEY_DOWN))
            {
                switch (ResultSelection)
                {
                    case 0:
                        DisplayResults();
                        break;
                    case 1:
                        DisplayGraphResults(SYSTOLIC_VALUES);
                        break;
                    case 2:
                        DisplayGraphResults(DYASTOLIC_VALUES);
                        break;
                    case 3:
                        DisplayGraphResults(HEARTBEAT_VALUES);
                        break;
                    default:
                        break;

                }               
            }

          
        }
        

        KeyHasBeenPressed = FALSE;
    }

     // If GUI (PC) call for data dump
    else if(SCI2_Interrupt==true && IS_USB_ON()==TRUE)
    {
        gu8MenuPrevState = gu8MenuActualState;
        gu8MenuActualState = MENU_STATE_DISPLAY_SENT;
    }
    
    
}

void vfnMenuStateMeasuring(void)
{
    if(gu8MenuPrevState == MENU_STATE_FIRST)
    {
        OLED_CLEAR_SCREEN();
        gu8MenuPrevState = gu8MenuActualState;
    }
    
    //if pressed enter key while measuring
    if ( (KeyHasBeenPressed && KeyPressed == KEY_ENTER))
    {
        stopMeasurement = BPMERROR;
        KeyHasBeenPressed = FALSE;        
    }


    //if measurement finished
    if (stopMeasurement == BPMERROR)
    {      
        pfBP = &vfnBloodPressureStop;
        gu8MenuActualState = MENU_STATE_ERROR;
        gu16DisplayDelay = 0;
        MenuDisplayError();
        return;
    }
    else if (stopMeasurement == BPMDONE)
    {        
        gu8MenuActualState = MENU_STATE_RESULTS;
    }
    
    if(gu16DisplayDelay>=200)
    {
        gu16DisplayDelay = 0;
        DisplayActualPressure();      
    }
    //update
    DisplayHeartBeating();
  
}
void SMAC_Constructor_BloodPresure(UINT8 u8type,UINT8 u8Data)
{
    if(u8type==SYS)    gau8TxBuff[1]= 2;
    if(u8type==DIA)    gau8TxBuff[1]= 5;
    if(u8type==PUL)    gau8TxBuff[1]= 8;

    gau8TxBuff[0]= 49;
    gau8TxBuff[2]= 0;
    gau8TxBuff[3]= u8Data; // Data To transmit
}
